' Syderal (space shooter game)
' Copyright (C) 2000, 2016-2017  Lucas Pedrosa

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, version 3 of the License.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' COMPILATION:
' Syderal has been compiled with FreeBasic 1.05.0 for GNU/Linux
' and Windows. It should compile well with any newer version
' and likely, with some older versions. You can get FreeBasic at:
' - http://www.freebasic.net
' For GNU/Linux, compile with:
' - fbc syderal.bas syderal.xpm
' For Windows, compile with:
' - fbc -s gui syderal.bas syderal.rc

Declare Sub Cycle
Declare Sub DemoGame
Declare Sub DisplayFigure(x As Short, y As Short, figure As Long)
Declare Sub ExecI
Declare Sub GenInit
Declare Function GrabJButton As Byte
Declare Sub HighScoreTable
Declare Sub Information
Declare Sub KillBug(x As Short, y As Short)
Declare Sub LevelSplash (n As Short)
Declare Sub LoadConfig
Declare Sub LoadGraphics
Declare Sub LoadHighScore
Declare Function LoadLevel (levelname As String) As String
Declare Sub LoadOrSave(which As Byte)
Declare Sub LoadTheGame(s As String)
Declare Function Menu As Short
Declare Function NewMenu(curop As Byte) As Short
Declare Sub MovePeople
Declare Sub Options
Declare Sub PalGrade (how As Byte)
Declare Sub Panel
Declare Sub Pause
Declare Function PlayGame (restart As Byte) As Byte
Declare Sub Present
Declare Sub PushScore (sig As String, score As Long, level As Short)
Declare Sub PutImage (x As Short, y As Short, n As Short)
Declare Sub SaveConfig
Declare Sub SaveHighScore
Declare Sub SaveTheGame(s As String)
Declare Sub SayGoodbye (n As Short)
Declare Sub SayHello (x As Short, y As Short, behaviour As UByte, id As Short, inistat As Long)
Declare Sub SetPalette(whichpal As Short, intensity As Short)
Declare Sub Shoot (x As Short, y As Short, d As Byte, kind As Short, victim As Short)
Declare Sub ShowScore (checklives As Byte)
Declare Sub SoundEvent (n As Short)
Declare Sub SoundMaintain
Declare Sub StartAni (x As Short, y As Short, firstframe As Short, frames As Short, speed As Short)
Declare Function TestBeams As Byte
Declare Sub TextDisp (x As Short, y As Short, text As String, col As UByte = 0)

#include "file.bi"
#include "fbgfx.bi"

'#define CLASSIC_STYLE   'Make it more like the original Syderal
#define _ENGLISH -1
#define _SPANISH
#define DEFAULT_LANGUAGE _ENGLISH

#define PANEL_IMAGE 1
#define SHOT_ICONS 2
#define NRG_ICONS 3
#define SCORE_DIGITS 4
#define SHIP_ICON 5
#define COLOURPALETTE 6
#define BNWPALETTE 7
#define SHOT_LOADING 8
#define SPRITES 9
#define SEQUENCE_ 10
#define SYDERAL_TITLE 11
#define PAUSE_BOX 12
#define PAUSE_ANI 13
#define MENU_LEFT 14
#define MENU_TOP 15
#define MENU_BOTTOM 16
#define MENU_OPTIONS 17
#define LEVEL_NUMBERS 18
#define LEVEL_SPLASH_TEXT 19
#define LEVEL_SCORE_LIVES 20
#define SAVEGAME_NUMBERS 21
#define DISKS 22
#define SKY_IMAGE 23
#define OPTIONS_SIGN 24
#define VOLUME_SETUP 25
#define LEVER_IMAGE 26
#define BRIGHTNESS_SETUP 27
#define JOYSTICK_DELTA 28
#define JOYSTICK_BOX 29
#define SND_BOOM 30
#define SND_SHOO 31
#define SND_CLANG 32
#define SND_ONEUP 33
#define SND_BUZZ 34
#define SND_UOHOH 35
#define SND_UZZ 36
'---------- PLUS
#define B_SPMENU 37
#define B_ENMENU 38
#define B_MLEFT 39
#define B_DIMILABS 40
#define B_SPLASH 41
#define B_SPLASH_PALETTE 42
#define B_SCORE_FONT 43
#define B_SND_CLAC 44
#define B_JBUTTONBOX 45
#define B_JBUTTONS 46
#define B_PATCH 47
#define B_ENLEVEL_SPLASH_TEXT 48
#define B_ENOPTIONS_SIGN 49
#define B_NEW_PAUSA 50
#define B_NEW_PAUSE 51
#define B_NEW_PAUSE_ANI 52
#define B_ENJBUTTONS 53
#define B_ENVOLUME_TITLE 54
#define B_ENBRIGHT_TITLE 55

#ifdef __FB_LINUX__
	#include "syderal_linux_static_sound.bas"	'Use static ALSA
	'#include "syderal_linux_sound.bas"			'Use dynamic ALSA
#elseif defined (__FB_WIN32__)
	#include "syderal_windows_sound.bas"
#else
	Type SoundChannel As Short
#endif

'Sound channels and their respective status
Type FXChannel
	id As SoundChannel		'Channel carrier (depends on O.S.)
	status As Long			'0 for available. Non zero for tics left
End Type
Dim Shared fx(1 To 10) As FXChannel

'Books are memory pages where images, palettes and sounds are loaded
Type ABook
	p As UByte Ptr		'Pointer to book
	id As String		'Book name
	length As Long		'Data length
End Type
Dim Shared book(1 To 60) As ABook, books As Short = 0

Type AnEnemy Field = 1
	x As Short
	y As Short
	d As Short
	nfirst As Short
	afirst As Short
	nframes As Short
	aframes As Short
	nspeed As Short
	aspeed As Short
	position As Short
	status As Short
	behaviour As Short
	weapon As Short
	prize As Short
	id As Short
	extra1 As Short
	extra2 As Short
	extra3 As Short
	extralong As Long
End Type
Dim Shared enemy(1 To 20) As AnEnemy

Type Animation Field = 1
	x As Short
	y As Short
	firstframe As Short
	lastframe As Short
	position As Short
	speed As Short
	status As Short
End Type
Dim Shared ani(1 To 20) As Animation

Type Beam Field = 1
	x As Short
	y As Short
	d As Short
	firstframe As Short
	lastframe As Short
	position As Short
	anispeed As Short
	shotspeed As Short
	status As Short
	victim As Short
	damage As Short
End Type
Dim Shared beam(1 To 20) As Beam

Type Star
	x As Short
	y As Short
	c As Byte
End Type
Dim Shared star(1 To 20) As Star

Type Ship Field = 1
	x As Short
	d As Short
	nrg As Short
	score As Long
	head As Single		'Called "puntero" in original source code
	record As Short
	shot As Short
	lives As Short
	level As Short
	cycle As Long
End Type
Dim Shared player As Ship

Type JType
	action As UByte		'Button number for: shoot/enter/action
	deal As UByte		'Button number for: deal combination
	start As UByte		'Button number for: pause/start
	back As UByte		'Button number for: back/exit/cancel
	config As Byte		'Joystick has been set up
	use As Byte			'Joystick enabled
	id As Byte			'Joystick number being used
End Type
Dim Shared joystick As JType

'This type is used within the load/save routines
Type SaveGame
	level As Short
	lives As Short
	score As Long
End Type

'This type is for highscore table entries
Type HScore
	signature As String * 12	'Player's signature
	level As Short				'Last level reached
	score As Long				'Record score
End Type
Dim Shared HighScore(1 To 10) As HScore

'Game constants
Const UNUSED As Short = &HFFF0, NORMAL As Short = Not UNUSED
Const ATTACK = 1, STANDBY = 2
Const DECAY = 3, UP = -1, DOWN = 1, MOVING = 7
Const PLAYERS = 1, ALLBUTME = 100, EVERYBODY = 3

Dim Shared errortext As String, errorflag As Short
Dim Shared bnw As Byte, playing As Byte, English As Byte = DEFAULT_LANGUAGE
Dim Shared curlev As String, levptr As Short, levmem As Long
Dim Shared currentm As Short, levels As Short
Dim Shared frequency As Short, QTH As String, brightness As Short
Dim Shared iscd As Byte, minibox As UByte Ptr
Dim Shared volume As Short, pal As UByte Ptr, splash As Byte
Dim Shared demomode As Byte, fullscreen As Byte, canscore As Byte

brightness = 5
volume = 5
frequency = 200
QTH = "syderal.qth"

GenInit		'General initialisation
LoadGraphics
minibox = ImageCreate(60, 60)
pal = Allocate(768)
SetPalette 0, -1		'Default palette at config brightness
Present

Dim n As Short = 1

Do
	'Pass menu position (1 to 7) and return absolute menu (0 to 8)
	n = NewMenu(n)
	Select Case n
		Case 0		'New game
			canscore = -1		'Player may record a highscore
			PlayGame -1
			If currentm = 255 Then Exit Do
			If playing Then n = 2 Else n = 1
		Case 1		'Continue game
			PlayGame 0
			If currentm = 255 Then Exit Do
			If playing Then n = 2 Else n = 1
		Case 2		'Save game
			LoadOrSave -1
			If currentm = 255 Then Exit Do
			n = 2
		Case 3		'Load game
			LoadOrSave 0
			If currentm = 255 Then Exit Do
			n = 4
		Case 4		'Demo
			demomode = -1
			DemoGame
			demomode = 0
			n = 2
		Case 5		'Highscores
			HighScoreTable
			n = 3
		Case 6		'Options
			Options
			If currentm = 255 Then Exit Do
			n = 5
		Case 7		'Info
			Information
			n = 6
		Case 8		'Exit
			Exit Do
		Case Is > 100		'Demo because of timeout
			demomode = -1
			DemoGame
			demomode = 0
			n = n - 100
	End Select
Loop

'-- Unload graphics, palettes, sounds and animations from memory
Deallocate pal
ImageDestroy minibox
For i As Short = 1 To books
	Select Case UCase(Right(book(i).id, 4))
		Case ".SSG", ".LIG", ".SGT", ".FNT"
			ImageDestroy book(i).p
		Case Else
			Deallocate book(i).p
	End Select
Next i
End


Sub Cycle
	Dim i As Byte, o As Byte, n As Short, m As Short
	Dim As Short y, exy
	
	SoundMaintain
	
	ScreenLock
	
	'Maintain splash
	Line (0, 0)-(959, 540), 0, BF
	If splash Then LevelSplash 1
			
	'Maintain stars
	For i = 1 To 20
		Line (3 * star(i).x, 3 * star(i).y)- Step (2, 2), star(i).c, BF
		If star(i).y >= 179 Then
			star(i).y = 0
			star(i).x = Int(Rnd * 320)
			star(i).c = Int(Rnd * 14)
		Else
			star(i).y += 1
		End If
	Next i
	
	'Maintain animations
	For i = 1 To 20
		If ani(i).status = Not UNUSED Then
			n = ani(i).firstframe + ani(i).position \ ani(i).speed
			If n > ani(i).lastframe Then
				ani(i).status = UNUSED
				'KillBug ani(i).x, ani(i).y
			Else
				PutImage ani(i).x, ani(i).y, n
				ani(i).position = ani(i).position + 1
			End If
		End If
	Next i
	
	'Maintain shots
	For i = 1 To 20
		If beam(i).status = Not UNUSED Then
			n = beam(i).firstframe + beam(i).position \ beam(i).anispeed
			PutImage beam(i).x, beam(i).y, n
			beam(i).position = beam(i).position + 1
			If n = beam(i).lastframe Then beam(i).status = MOVING: beam(i).position = 0
		ElseIf beam(i).status = MOVING Then
			exy = beam(i).y + beam(i).d * 2 * beam(i).position \ beam(i).shotspeed
			'Line (3 * beam(i).x, 3 * exy)-(3 * beam(i).x + 59, 3 * exy + 59), 0, BF
			beam(i).position = beam(i).position + 1
			y = beam(i).y + beam(i).d * 2 * beam(i).position \ beam(i).shotspeed
		   
			If y > 139 And Abs(beam(i).x - player.x) < 14 Then
				player.nrg = player.nrg - beam(i).damage
				IF player.nrg < 0 THEN player.nrg = 0
				beam(i).status = UNUSED
				SoundEvent 3
			ElseIf y < 0 Or y > 159 Then
				beam(i).status = UNUSED
			Else
				PutImage beam(i).x, y, beam(i).lastframe
				If beam(i).victim = EVERYBODY Then
					For o = 1 To 20
						IF enemy(o).status <> UNUSED And Abs(enemy(o).x - beam(i).x) < 18 And Abs(enemy(o).y - y) < 18 Then
							beam(i).status = UNUSED
							enemy(o).status = UNUSED
							enemy(o).id = 0
							'Line (3 * beam(i).x, 3 * y)-(3 * beam(i).x + 59, 3 * y + 59), 0, BF
							SoundEvent 1
							StartAni enemy(o).x, enemy(o).y, 30, 6, 12
							   
							player.score = player.score + enemy(o).prize
						End If
					Next o
				End If
			End If
		End If
	Next i
	
	'Manage enemies
	For i = 1 To 20
		If enemy(i).status = STANDBY Then
			If enemy(i).position > 0 Then
				enemy(i).position = enemy(i).position - 1
			Else
				enemy(i).status = NORMAL
			End If
		ElseIf enemy(i).status = NORMAL Then
			n = enemy(i).nfirst + enemy(i).position \ enemy(i).nspeed
			m = enemy(i).nframes + enemy(i).nfirst - 1
		   
			'KillBug enemy(i).x, enemy(i).y
			PutImage enemy(i).x, enemy(i).y, n
		   
			enemy(i).position = enemy(i).position + 1
			If n = m Then enemy(i).position = 0
		ElseIf enemy(i).status = ATTACK Then
			n = enemy(i).afirst + enemy(i).position \ enemy(i).aspeed
			m = enemy(i).aframes + enemy(i).afirst - 1

			'KillBug enemy(i).x, enemy(i).y
			PutImage enemy(i).x, enemy(i).y, n
		  
			enemy(i).position = enemy(i).position + 1
			If n = m Then enemy(i).status = DECAY: Shoot enemy(i).x, enemy(i).y + 20, DOWN, enemy(i).weapon, PLAYERS
		ElseIf enemy(i).status = DECAY Then
			n = enemy(i).afirst + enemy(i).position \ enemy(i).aspeed
		  
			'KillBug enemy(i).x, enemy(i).y
			PutImage enemy(i).x, enemy(i).y, n
		 
			enemy(i).position = enemy(i).position - 1
			If n = enemy(i).afirst Then enemy(i).status = NORMAL
		End If
	Next i
	
	'Draw ship
	PutImage player.x, 160, player.d
		
	ScreenUnlock

	'This is the AI
	MovePeople
End Sub


Sub DemoGame
	Dim As Short exx, d, lastcycle
	Dim reason As Byte, splash As Byte
	Dim lasttimer As Double, remember As Double
	Dim levelon As Short, lv As String
	Dim levelwait As Short, excuseme As Byte
	Dim t As Double, timetoturn As Double, turn As Short
	Dim jb As Integer, jx As Single, jy As Single

	If iscd = 0 And playing <> 0 Then SaveTheGame "current.sav"

	Do : Loop Until Len(InKey) = 0
	Randomize Timer

	player.x = 150
	exx = 151 : d = 0
	player.score = 0
	player.head = 0
	player.nrg = 7
	player.shot = 70
	player.lives = 0
	player.record = 0
	If QTH = "syderal.qth" Then
		player.level = Rnd * 20
	Else
		player.level = 0
	End If
	player.cycle = 0
	
	curlev = ""		'Instruction buffer
	levptr = 1		'Pointer to current instruction
	levmem = 0		'Sleep time before executing next instruction
	
	For i As Byte = 1 To 20
		enemy(i).status = UNUSED
		ani(i).status = UNUSED
		beam(i).status = UNUSED
	Next i
	
	lastcycle = player.cycle
	lasttimer = Timer
	
	lv = Trim(Str(player.level))
	lv = String(3 - Len(lv), "0") + lv
	curlev = LoadLevel(lv)
	splash = -1
	levmem = 500
	LevelSplash 1
	
	Do
		CLS
		ShowScore -1
		Do
			t = Timer
			
			'Every 500 points, player gets a life
			If player.head >= player.record + 500 And player.lives < 7 Then
				player.record = player.head
				player.lives += 1
				ShowScore -1
				SoundEvent 5
			End If
			
			player.cycle += 1
			If levmem > 0 Then levmem -= 1
			
			'player.shot is negative when charging and is used to count
			If player.shot <= 0 Then
				player.shot -= 1
				If player.shot = -1800 Then player.shot = 70
			End If
			
			'Have all data been executed for current level?
			If levptr > Len(curlev) Then
				player.level += 1	'Yes. Go to next level
				
				'Wrap around when all levels have been played
				'levelon is the actual level been loaded
				levelon = ((player.level - 1) Mod levels) + 1
				lv = Trim(Str(levelon))
				lv = String(3 - Len(lv), "0") + lv
				curlev = LoadLevel(lv)
				
				'Initialise new level
				player.shot = 70
				player.cycle = 0
				lastcycle = 0
				lasttimer = Timer
				levptr = 1
				splash = -1
				
				If TestBeams Then
					levmem = 1500
					levelwait = -1
				Else
					levmem = 500
					LevelSplash 1
				End If
			End If
			
			If levelwait <> 0 Then
				If TestBeams = 0 Then
					levmem = 500
					LevelSplash 1
					levelwait = 0
				End If
			End If
			
			If levptr = 1 And levmem = 0 Then LevelSplash 0 : splash = 0
			
			'Draw ship
			If player.x <> exx Then
				Line (exx * 3, 480)-(exx * 3 + 59, 539), 0, BF
				PutImage player.x, 160, d
				exx = player.x
				remember = Timer
			ElseIf d <> 0 And Timer - remember >= .05 Then
				PutImage player.x, 160, 0
				d = 0
			End If
									
			'Check on key press
			If Len(InKey) Then player.nrg = 0
			
			'Check on joystick use
			If joystick.use Then
				GetJoystick joystick.id, jb, jx, jy
				
				If jb <> 0 Or jx <> 0 Or jy <> 0 Then player.nrg = 0
			End If
		
			'Make ship shoot from time to time
			If Int(Rnd * 300) = 1 Then
				If player.shot > 0 And splash = 0 Then
					Shoot player.x, 140, UP, 1, EVERYBODY
					player.shot -= 1
				End If
			End If
			
			'Make ship go back and forth
			If Timer > timetoturn Then
				If turn Then
					player.x -= 1 : d = 2
					If player.x = -1 Then player.x = 1 : turn = 0 : d = 1
				Else
					player.x += 1 : d = 1
					If player.x = 300 Then player.x = 298 : turn = -1 : d = 2
				End If
			End If
		
			Cycle
			ShowScore 0
			
			If player.head < player.score Then player.head += .1
			
			'Sync to 200 cycles per second
			Do : Loop Until Timer >= t + .005
			
			If levmem = 0 And levptr <= Len(curlev) Then ExecI
		Loop Until player.nrg <= 0
		
		If player.nrg <= 0 Then
			StartAni player.x, 160, 30, 6, 12
			SoundEvent 1
			player.x = -50	'So shots aren't detected anymore
			
			Dim tp(768) As UByte, i As Short, which As Short, n As Short
			If bnw Then which = BNWPALETTE Else which = COLOURPALETTE
			
			For i = 0 To 767
				tp(i) = book(which).p[i]
			Next i
			
			For n = 1 To 600
				t = Timer
				Cycle
				
				If n > 400 Then
					For i = 0 To 767
						If tp(i) > n - 400 Then
							tp(i) -= (n - 400) \ 2
						Else
							tp(i) = 0
						End If
					Next i
					For i = 0 To 765 Step 3
						Palette i \ 3, tp(i) ShL 2, tp(i + 1) ShL 2, tp(i + 2) ShL 2
					Next i
				End If
				
				Do : Loop Until Timer >= t + .008
			Next n
			player.lives -= 1
			If player.lives = -1 Then reason = 1: Exit Do
			player.x = 150 : d = 0
			exx = 151
			player.nrg = 7
			player.shot = 70
			lastcycle = 0
			lasttimer = Timer
			player.level -= 1			'These two lines force
			curlev = "" : levptr = 1	'level restart
			
			For i = 1 To 20
				enemy(i).status = UNUSED
				beam(i).status = UNUSED
			Next i
			CLS
			PalGrade 3
		End If
	Loop
	
	If iscd = 0 And playing <> 0 Then
		LoadTheGame "current.sav"
		Kill "current.sav"
	End If
End Sub


Sub DisplayFigure(x As Short, y As Short, figure As Long)
	Dim imtemp As UByte Ptr
	Dim num As String, n As Byte
	Dim As Short xoffset, yoffset
	
	imtemp = ImageCreate(30, 36)
	
	num = Trim(Str(figure))
	xoffset = x - Len(num) * 30
	yoffset = y
	For i As Short = 1 To Len(num)
		n = ValInt(Mid(num, i, 1))
		Get book(SAVEGAME_NUMBERS).p, (0, 36 * n)- Step (29, 35), imtemp
		Put (xoffset, yoffset), imtemp, PSet
		xoffset += 30
	Next i
	
	ImageDestroy imtemp
End Sub


Sub ExecI
	Select Case ASC(Mid(curlev, levptr, 1))
		Case 255	'End All
			For k As Byte = 1 To 20
				If enemy(k).status <> UNUSED Then SayGoodbye k
			Next k
			levmem = 500
			levptr += 1
		Case 1		'Wait <t>
			levmem = CvShort(Mid(curlev, levptr + 1, 2))
			levptr += 3
		Case 2		'Start <x, y> <o> as <id>
			Dim As Short x, y, b, i
			x = CvShort(Mid(curlev, levptr + 1, 2))
			y = CvShort(Mid(curlev, levptr + 3, 2))
			b = CvShort(Mid(curlev, levptr + 5, 2))
			i = CvShort(Mid(curlev, levptr + 7, 2))
			
			SayHello x, y, b, i, &HFFFFFF
			levptr += 9
		Case 3		'End <id>
			Dim As Short i, k
			i = CvShort(Mid(curlev, levptr + 1, 2))
			
			For k = 1 To 20
				If enemy(k).id = i Then SayGoodbye k
			Next k
			levptr += 3
		Case 4		'Start <x,y> <o> <inistat> as <id>
			Dim As Short x, y, b, i
			Dim inistat As Long
			x = CvShort(Mid(curlev, levptr + 1, 2))
			y = CvShort(Mid(curlev, levptr + 3, 2))
			b = CvShort(Mid(curlev, levptr + 5, 2))
			i = CvShort(Mid(curlev, levptr + 7, 2))
			inistat = CvL(Mid(curlev, levptr + 9, 4))
			
			SayHello x, y, b, i, inistat
			levptr += 13
		Case Else	'Unknown instruction
			levptr += 1
	End Select
End Sub


Sub GenInit
	Dim f As Integer
	
	f = FreeFile
	Screen 12

	If Not FileExists("syderal.scc") Then
		Print
		If English Then
			Print "Could not find 'syderal.scc'"
		Else
			Print "No se encontr" + Chr(162) + " 'syderal.scc'"
		End If
		Print
		GetKey
		End
	ElseIf Not FileExists("plus.scc") Then
		Print
		If English Then
			Print "Could not find 'plus.scc'"
		Else
			Print "No se encontr" + Chr(162) + " 'plus.scc'"
		End If
		Print
		GetKey
		End
	End If
	
	'Make sure there's a configuration file
	If Not FileExists("syderal.cfg") Then
		If iscd Then
			Color 7
			If English Then
				Print "Could not find 'syderal.cfg'"
			Else
				Print "No se encontr" + Chr(162) + " 'syderal.cfg'"
			End If
			GetKey
			End
		Else
			Open "syderal.cfg" For Output As f
			Print #f, "B&W=OFF"
			Print #f, "BRIGHT=5"
			Print #f, "JOYSTICK=PS"
			Close f
		End If
	End If
	LoadConfig
	Screen 0	'Necessary for ScreenInfo and ScreenRes to calculate well
	
	If fullscreen Then
		Dim As Integer swidth, sheight
		
		ScreenInfo swidth, sheight
		If fullscreen = -2 Then
			ScreenRes swidth, sheight, , , FB.GFX_NO_FRAME + FB.GFX_ALWAYS_ON_TOP
		Else
			ScreenRes swidth, sheight, , , FB.GFX_FULLSCREEN
		End If
		Width swidth \ 8, sheight \ 16
	Else
		'This is 320x200 scaled by 3
		ScreenRes 960, 600, 8
		Width 960 \ 8, 600 \ 16		'8x16 character box
		WindowTitle "Syderal"
	End If
	
	Color 10
	Print
	Print "Syderal - Dimioca Labs"
	Color 2
	Print "Copyright(c) 2000, 2016-2017 Lucas Pedrosa"
	If English Then
		Print "First built in 2000"
		Print "Modern OS adaptation: May 2016"
		Print
		Print "Syderal is FREE SOFTWARE. You can redistribute it and/or"
		Print "modify it under the terms of the GNU General Public License,"
		Print "version 3, as published by the Free Software Foundation."
		Print
		Print "This program is distributed in the hope that it will be useful,"
		Print "but WITHOUT ANY WARRANTY; without even the implied warrany of"
		Print "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the"
		Print "GNU General Public License for more details."
		Print
		Print "You should have received a copy of the GNU General Public Licence"
		Print "along with this program. If not, see <http://www.gnu.org/linceses/>."
		Print
		Print "Initialising..."
	Else
		Print "Versi" + Chr(162) + "n original - 2000"
		Print "Adaptado a sistemas modernos en 2016"
		Print
		Print "Syderal es SOFTWARE LIBRE. Puede redistribuir y/o modificar"
		Print "este programa bajo los t" + Chr(130) + "rminos de la Licencia General P" + Chr(163) + "blica"
		Print "de GNU (GPL), versi" + Chr(162) + "n 3, tal como la publica la Free Software"
		Print "Foundation."
		Print
		Print "Se distribuye este programa en espera de que sea " + Chr(163) + "til, pero"
		Print "SIN NINGUNA GARANTIA; ni siquiera la garant" + Chr(161) + "a impl" + Chr(161) + "cita de"
		Print "MERCANTABILIDAD o ADECUACION PARA UN PROPOSITO PARTICULAR."
		Print "Vea la Licencia General P" + Chr(163) + "blica de GNU para m" + Chr(160) + "s detalles."
		Print
		Print "Usted deber" + Chr(161) + "a haber recibido una copia de la Licencia General"
		Print "P" + Chr(163) + "blica junto con este programa. De no ser as" + Chr(161) + ", visite"
		Print "<http://www.gnu.org/licenses/>."
		Print
		Print "Inicializando el juego..."
	End If
	
	'Check if the game is being run from a writable disk
	If Open("test.tmp" For Output As f) Then
		iscd = -1	'Cannot write to disk
	Else
		Close f
		Kill "test.tmp"
		iscd = 0
	End If
		
	'See if a user level is specified
	Dim q As Short
	q = InStr(UCase(Command), "-Q")
	If q Then
		QTH = Trim(Mid(Command, q + 2))
		q = InStr(QTH, " ")
		If q Then QTH = Left(QTH, q - 1)
		If InStr(QTH, ".") = 0 Then QTH = QTH + ".qth"
	End If
	
	'Sound detection
	#ifdef __FB_LINUX__
		If alsa Then
			If English Then
				Print "ALSA sound system detected..."
			Else
				Print "Se detect" + Chr(162) + " sonido ALSA..."
			End If
			For i As Byte = 1 To 10
				ChannelInit @fx(i).id
				fx(i).status = 0
			Next i
		End If
	#elseif defined (__FB_WIN32__)
		If English Then
			Print "Initialising Windows sound..."
		Else
			Print "Inicializando sonido de Windows..."
		End If
		For i As Byte = 1 To 10
			ChannelInit @fx(i).id
			fx(i).status = 0
		Next i
	#endif
	
	'If joystick has not been inhibited, try to detect it
	joystick.id = -1
	If InStr(LCase(Command), "-j") = 0 Then
		For i As Short = 0 To 15
			If GetJoystick(i) = 0 Then
				joystick.id = i
				joystick.use = -1
				If joystick.config = 0 Then
					joystick.action = 0
					joystick.deal = 1
					joystick.start = 2
					joystick.back = 3
				End If
				If English Then
					Print "Joystick/gamepad detected..."
				Else
					Print "Se detect" + Chr(162) + " joystick..."
				End If
				Exit For
			End If
		Next i
	End If
	
	If Not FileExists(QTH) Then
		Color 7
		If English Then
			Print "Could not find '" + QTH + "'"
		Else
			Print "No se encontr" + Chr(162) + " '" + QTH + "'"
		End If
		GetKey
		End
	End If
	Open QTH For Binary Access Read As f
	Get #f, 1, levels
	Close f
	If QTH <> "syderal.qth" Then Print "Usando " + QTH + "..."
	
	Randomize Timer
	
	Dim i As Short
	For i = 1 To 20
		enemy(i).status = UNUSED
		ani(i).status = UNUSED
		beam(i).status = UNUSED
	Next i
	
	If English Then
		Print "Press any key to continue...    ";
	Else
		Print "Presione cualquier tecla para continuar...    ";
	End If
	
	Dim t As Double, show_cursor As Boolean
	t = Timer
	Do
		Sleep 250
		Locate , Pos(0) - 3
		Print Trim(Str(Int(6 + t - Timer))) & "s";
		If show_cursor Then Print Chr(219); Else Print " ";
		show_cursor = Not show_cursor
	Loop Until Len(InKey) <> 0 Or Timer > t + 5
	Locate , Pos(0) - 1 : Print " "
		
	If English Then
		Print "Scaling graphics..."
	Else
		Print "Escalando gr" + Chr(160) + "ficos..."
	End If
End Sub


Function GrabJButton As Byte
	Dim jb As Integer, akey As String
	
	Do : Loop Until Len(InKey) = 0
	
	Do
		GetJoystick joystick.id, jb
		akey = InKey
		
		If Len(akey) Then Return 0
	Loop Until jb = 0
	
	Do
		GetJoystick joystick.id, jb
		akey = InKey
		
		If Len(akey) Then Return 0
	Loop Until jb <> 0
	
	For i As Byte = 0 To 11
		If jb And 2 ^ i Then Return i + 1
	Next i
	
	Return 0
End Function


Sub HighScoreTable
	Dim s As String, num As String, jb As Integer
	
	Put (0, 0), book(B_MLEFT).p, PSet
	Put (240, 0), book(SKY_IMAGE).p, PSet
	
	LoadHighScore
	
	If English Then
		TextDisp 414, 30, "HIGHSCORE TABLE", 1
	Else
		TextDisp 390, 30, "TABLA DE PUNTAJES", 1
	End If
	For i As Byte = 1 To 10
		s = Trim(Str(i))
		If Len(s) = 1 Then s = " " + s
		s &= ". " + HighScore(i).signature
		num = Trim(Str(HighScore(i).level))
		If Len(num) = 1 Then num = " " + num
		s &= " " + num + " "
		num = Trim(Str(HighScore(i).score))
		num = Space(6 - Len(num)) + num
		s &= num
		TextDisp 270, 39 * i + 90, s
	Next i
	
	PalGrade 1
	
	Do : Loop Until Len(InKey) = 0
	
	Do
		s = InKey
		
		If joystick.use Then
			GetJoystick joystick.id, jb
			If jb And 2 ^ joystick.action Then
				s = Chr(13)
			ElseIf jb And 2 ^ joystick.back Then
				s = Chr(27)
			End If
		End If
	Loop Until Len(s)
	
	PalGrade 2
	CLS
	PalGrade 3
End Sub


Sub Information
	Dim y As Short, s As String, jb As Integer, jlock As Byte
	Dim update As Byte, p As Byte
	
	Put (0, 0), book(B_MLEFT).p, PSet
			
	PalGrade 1
	
	Do : Loop Until Len(InKey) = 0
	
	update = -1
	Do
		If update Then
			ScreenLock
			Put (240, 0), book(SKY_IMAGE).p, PSet
			
			Select Case p		'Information page
				Case 0
					y = 90
					If English Then
						TextDisp -1, y, "SYDERAL 1.6", 2 : y += 78
						TextDisp -1, y, "BY LUCAS PEDROSA" : y += 39
						TextDisp -1, y, "FIRST BUILT IN 2000" : y += 39
						TextDisp -1, y, "ADAPTATION MADE IN MAY 2016" : y += 78
						TextDisp -1, y, "SOFTWARE LICENSE" : y += 39
						TextDisp -1, y, "G.P.L. V3", 3 : y += 78
						TextDisp -1, y, "TO MOM", 4					
					Else
						TextDisp -1, y, "SYDERAL 1.6", 2 : y += 78
						TextDisp -1, y, "POR LUCAS PEDROSA" : y += 39
						TextDisp -1, y, "VERSI" + Chr(4) + "N ORIGINAL EN 2000" : y += 39
						TextDisp -1, y, "REEDITADO EN MAYO DE 2016" : y += 78
						TextDisp -1, y, "LICENCIA DE DISTRIBUCI" + Chr(4) + "N" : y += 39
						TextDisp -1, y, "G.P.L. V3", 3 : y += 78
						TextDisp -1, y, "PARA MAM" + Chr(1), 4
					End If
				Case 1
					y = 120
					If English Then
						TextDisp -1, y, "CONTROLS", 5 : y += 78
						TextDisp 528, y, "UP", 4
						TextDisp 600, y, "SHOOT" : y += 39
						TextDisp 480, y, "DOWN", 4
						TextDisp 600, y, "ENERGY DEAL" : y += 39
						TextDisp 504, y, "DEL", 4
						TextDisp 600, y, "GIVE UP LIFE" : y += 39
						TextDisp 552, y, "P", 4
						TextDisp 600, y, "PAUSE" : y += 39
						TextDisp 504, y, "F10", 4
						TextDisp 600, y, "SUICIDE" : y += 39
						TextDisp 504, y, "ESC", 4
						TextDisp 600, y, "MENU"					
					Else
						TextDisp -1, y, "CONTROLES", 5 : y += 78
						TextDisp 432, y, "ARRIBA", 4
						TextDisp 600, y, "DISPARAR" : y += 39
						TextDisp 456, y, "ABAJO", 4
						TextDisp 600, y, "CANJE" : y += 39
						TextDisp 504, y, "DEL", 4
						TextDisp 600, y, "USAR VIDA" : y += 39
						TextDisp 552, y, "P", 4
						TextDisp 600, y, "PAUSA" : y += 39
						TextDisp 504, y, "F10", 4
						TextDisp 600, y, "SUICIDIO" : y += 39
						TextDisp 504, y, "ESC", 4
						TextDisp 600, y, "MENÚ"
					End If
			End Select
			ScreenUnlock
			
			update = 0
		End If
		
		s = InKey
		
		If joystick.use Then
			GetJoystick joystick.id, jb
			
			If jb = 0 Then jlock = 0
			
			If jlock = 0 Then
				If jb And 2 ^ joystick.action Then
					s = Chr(13) : jlock = -1
				ElseIf jb And 2 ^ joystick.back Then
					s = Chr(27) : jlock = -1
				End If
			End If
		End If
		
		Select Case s
			Case Chr(13), Chr(255) + Chr(77)
				p = (p + 1) Mod 2
				update = -1
			Case Chr(255) + Chr(75)
				If p > 0 Then p -= 1 Else p = 1
				update = -1
			Case Chr(27) : Exit Do
		End Select
	Loop
	
	PalGrade 2
	CLS
	PalGrade 3
End Sub


Sub KillBug(x As Short, y As Short)
	If y + 21 > 160 Then
		Line (3 * x - 6, 3 * y - 6)-(3 * x + 63, 539), 0, BF
	Else
		Line (3 * x - 6, 3 * y - 6)-(3 * x + 63, 3 * y + 63), 0, BF
	End If
End Sub


Sub LevelSplash (n As Short)
	Dim imtemp As UByte Ptr, lv As String
	Dim pixelpos As Short, figure As Byte
	
	If n Then
		imtemp = ImageCreate(108, 132)
		lv = Trim(Str(player.level))
		pixelpos = 219 - 54 * Len(lv)	'Since numbers are 108px wide
		If English Then
			Put (pixelpos, 165), book(B_ENLEVEL_SPLASH_TEXT).p, Trans
		Else
			Put (pixelpos, 165), book(LEVEL_SPLASH_TEXT).p, Trans
		End If
		pixelpos += 522
		For i As Byte = 1 To Len(lv)
			figure = ValInt(Mid(lv, i, 1))
			Get book(LEVEL_NUMBERS).p, (0, 132 * figure)- Step (107, 131), imtemp
			Put (pixelpos, 165), imtemp, Trans
			pixelpos += 108
		Next i
		ImageDestroy imtemp
	Else
		Line (0, 165)-(959, 369), 0, BF
	End If
End Sub


Sub LoadConfig
	Dim f As Integer, a As String, n As Short
	Dim As String cm, arg
	
	f = FreeFile
	Open "syderal.cfg" For Input As f
	
	Do While Not EOF(f)
		Line Input #f, a
		
		n = InStr(a, ";")
		If n Then a = Left(a, n - 1)
		a = Trim(a)
		
		n = InStr(a, "=")
		If n Then
			cm = Ucase(Trim(Left(a, n - 1)))
			arg = Trim(Mid(a, n + 1))
		Else
			cm = UCase(a)
			arg = ""
		End If
		
		If Len(a) Then
			Select Case cm
				Case "B&W", "BNW", "B&N", "BYN"
					If arg <> "0" And UCase(arg) <> "NO" And UCase(arg) <> "OFF" Then bnw = 1 Else bnw = 0
				Case "BRIGHT", "BRIGHTNESS", "BRILLO"
					brightness = ValInt(arg) Mod 11
					If brightness < 0 Then brightness = 5
				Case "JOY", "JOYSTICK"
					If Left(LCase(arg), 2) = "ps" Then
						joystick.action = 2	'"x" button
						joystick.deal = 5	'R1 button
						joystick.start = 9	'Start button
						joystick.back = 0	'Triangle button
						joystick.config = -1
					End If
				Case "JACTION", "JSHOOT"
					If ValInt(arg) >= 0 And ValInt(arg) <= 15 Then
						joystick.action = ValInt(arg)
						If joystick.action < 0 Or joystick.action > 11 Then joystick.action = 3
						joystick.config = -1
					End If
				Case "JDEAL"
					If ValInt(arg) >= 0 And ValInt(arg) <= 15 Then
						joystick.deal = ValInt(arg)
						If joystick.deal < 0 Or joystick.deal > 11 Then joystick.deal = 4
						joystick.config = -1
					End If
				Case "JSTART", "JPAUSE"
					If ValInt(arg) >= 0 And ValInt(arg) <= 15 Then
						joystick.start = ValInt(arg)
						If joystick.start < 0 Or joystick.start > 11 Then joystick.start = 11
						joystick.config = -1
					End If
				Case "JBACK", "JEXIT"
					If ValInt(arg) >= 0 And ValInt(arg) <= 15 Then
						joystick.back = ValInt(arg)
						If joystick.back < 0 Or joystick.back > 11 Then joystick.action = 1
						joystick.config = -1
					End If	
				Case "VIDEO", "SCREEN"
					Select Case Trim(LCase(arg))
						Case "fullscreen", "full", "fs", "completo"
							fullscreen = -1
						Case "noborder", "special", "especial"
							fullscreen = -2
						Case Else
							fullscreen = 0
					End Select
				Case "VOLUME", "VOL", "VOLUMEN"
					volume = ValInt(arg)
					If volume < 0 Or volume > 10 Then volume = 5
				Case "LANGUAGE", "LANG", "IDIOMA"
					Select Case Left(LCase(Trim(arg)), 2)
						Case "en", "in" : English = -1
						Case Else : English = 0
					End Select
			End Select
		End If
	Loop
	
	Close f
End Sub


Sub LoadGraphics
	Dim f As Integer
	Dim flength As Long, a As String, n As Short
	Dim As Long i, j, k, boff
	Dim As Short x, y, truewidth
	Dim frames As UByte
	Dim tempoint As UByte Ptr
	Dim nextdescriptor As Long
	
	f = FreeFile
	Open "syderal.scc" For Binary Access Read As f
	Do
		a = Space(21)
		Get #f, , a
		n = InStr(a, Chr(0))
		books += 1
		book(books).id = Left(a, n - 1)
		flength = CvL(Mid(a, n + 1, 4))
		
		Select Case Right(book(books).id, 4)
			Case ".SSG"		'Single image - scale
				Seek #f, Seek(f) + 4
				Get #f, , x
				Get #f, , y
				book(books).p = ImageCreate(3 * x, 3 * y)
				If (x And 15) = 0 Then
					truewidth = 3 * x
				Else
					truewidth = 3 * x + 16 - ((3 * x) And 15)
				End If
				
				tempoint = Allocate(3 * x)
				For j = 0 To 3 * y - 1 Step 3
					Get #f, , *tempoint, x
					For i = 3 * x - 1 To 1 Step -1
						tempoint[i] = tempoint[i \ 3]
					Next i
					For k = 0 To 2
						boff = truewidth * (j + k) + 32
						For i = 0 To 3 * x - 1
							book(books).p[boff + i] = tempoint[i]
						Next i
					Next k
				Next j
				Deallocate tempoint
			Case ".SGT"		'Header-less image - scale
				book(books).p = ImageCreate(720, 600)
				tempoint = Allocate(720)
				For j = 0 To 599 Step 3
					Get #f, , *tempoint, 240
					For i = 719 To 1 Step -1
						tempoint[i] = tempoint[i \ 3]
					Next i
					For k = 0 To 2
						boff = 720 * (j + k) + 32
						For i = 0 To 719
							book(books).p[boff + i] = tempoint[i]
						Next i
					Next k
				Next j
				Deallocate tempoint
			Case ".LIG"		'Animation - scale
				nextdescriptor = Seek(f) + flength
				Seek #f, Seek(f) + 4
				Get #f, , x
				Get #f, , y
				Get #f, , frames
				book(books).p = ImageCreate(3 * x, 3 * frames * y)
				tempoint = Allocate(3 * x)
				
				If (x And 15) = 0 Then
					truewidth = 3 * x
				Else
					truewidth = 3 * x + 16 - ((3 * x) And 15)
				End If
				
				For j = 0 To 3 * frames * y - 1 Step 3
					Get #f, , *tempoint, x
					For i = 3 * x - 1 To 1 Step -1
						tempoint[i] = tempoint[i \ 3]
					Next i
					For k = 0 To 2
						boff = truewidth * (j + k) + 32
						For i = 0 To 3 * x - 1
							book(books).p[boff + i] = tempoint[i]
						Next i
					Next k
				Next j
				Deallocate tempoint
				Seek #f, nextdescriptor
			Case Else		'Something else - leave as it is
				book(books).p = Allocate(flength)
				Get #f, , *book(books).p, flength
				book(books).length = flength
		End Select
	Loop Until Seek(f) > LOF(f)
	Close f
	
	'#ifndef CLASSIC_STYLE
	f = FreeFile
	Open "plus.scc" For Binary Access Read As f
	Seek #f, 7
	Do
		Seek #f, Seek(f) + 4	'Book number and encoding are ignored
		Get #f, , flength
		Get #f, , n
		a = Space(n)
		Get #f, , a
		
		books += 1
		book(books).id = a
				
		Select Case UCase(Right(book(books).id, 4))
			Case ".SSG", ".FNT"		'Single image - scale
				Seek #f, Seek(f) + 4
				Get #f, , x
				Get #f, , y
				book(books).p = ImageCreate(3 * x, 3 * y)
				If (x And 15) = 0 Then
					truewidth = 3 * x
				Else
					truewidth = 3 * x + 16 - ((3 * x) And 15)
				End If
				
				tempoint = Allocate(3 * x)
				For j = 0 To 3 * y - 1 Step 3
					Get #f, , *tempoint, x
					For i = 3 * x - 1 To 1 Step -1
						tempoint[i] = tempoint[i \ 3]
					Next i
					For k = 0 To 2
						boff = truewidth * (j + k) + 32
						For i = 0 To 3 * x - 1
							book(books).p[boff + i] = tempoint[i]
						Next i
					Next k
				Next j
				Deallocate tempoint
			Case ".LIG"		'Animation - scale
				nextdescriptor = Seek(f) + flength
				Seek #f, Seek(f) + 4
				Get #f, , x
				Get #f, , y
				Get #f, , frames
				book(books).p = ImageCreate(3 * x, 3 * frames * y)
				tempoint = Allocate(3 * x)
				
				If (x And 15) = 0 Then
					truewidth = 3 * x
				Else
					truewidth = 3 * x + 16 - ((3 * x) And 15)
				End If
				
				For j = 0 To 3 * frames * y - 1 Step 3
					Get #f, , *tempoint, x
					For i = 3 * x - 1 To 1 Step -1
						tempoint[i] = tempoint[i \ 3]
					Next i
					For k = 0 To 2
						boff = truewidth * (j + k) + 32
						For i = 0 To 3 * x - 1
							book(books).p[boff + i] = tempoint[i]
						Next i
					Next k
				Next j
				Deallocate tempoint
				Seek #f, nextdescriptor
			Case Else		'Something else - leave as it is
				book(books).p = Allocate(flength)
				Get #f, , *book(books).p, flength
				book(books).length = flength
		End Select
	Loop Until Seek(f) > LOF(f)
	Close f
	'#endif
End Sub


Sub LoadHighScore
	Dim hsunav As Byte, f As Integer
	Dim s As String, l As UByte
	Dim As Byte i, j

	If Not FileExists("syderal.hsc") Then
		hsunav = -1
	Else
		f = FreeFile
		Open "syderal.hsc" For Binary Access Read As f
		Seek #f, 9
		For i = 1 To 10
			Get #f, , l
			l XOr= 43
			If l > 12 Then hsunav = -1 : Exit For
			s = Space(l)
			Get #f, , s
			Get #f, , l		'Number to XOr it with
			For j = 1 To Len(s)
				Mid(s, j, 1) = Chr(ASC(Mid(s, j, 1)) XOr l XOr i)
			Next j
			HighScore(i).signature = s + Space(12 - Len(s))
			Get #f, , HighScore(i).score
			HighScore(i).score XOr= &HFEDECACA
			If HighScore(i).score < 0 Or HighScore(i).score > 999999 Then hsunav = -1 : Exit For
			Get #f, , HighScore(i).level
			HighScore(i).level XOr= &HDED0 XOr (i ShL 1)
			If HighScore(i).level < 1 Or HighScore(i).level > 20 Then hsunav = -1 : Exit For
			Seek #f, Seek(f) + 3
		Next i
		Close f
	End If
	
	If hsunav Then
		For i = 1 To 10
			HighScore(i).signature = "SYDERAL" + Space(5)
			HighScore(i).level = 1
			HighScore(i).score = 100
		Next i
	End If
End Sub


Function LoadLevel(levelname As String) As String
	Dim t As String, f As Integer
	Dim a As String, p As Long, n As Short
	
	t = "<L" + LCase(levelname) + ">"
	f = FreeFile
	
	'----- HAVE TO CHECK IF FILE EXISTS!!!!!
	
	Open QTH For Binary Access Read As f
	a = Space(5000)
	Do
		p = Seek(f)
		Get #f, , a
		n = InStr(a, t)
		If n Then	'Level was found in QTH file
			p = p + n - 1 : Exit Do
		Else		'Not found. Keep on searching
			Seek f, p + 5000 - Len(t)
		End If
		
		'If it wasn't found, just load the first level in the file
		If Seek(f) > LOF(f) Then
			t = "<L001>"
			p = 1
			Exit Do
		End If
	Loop
	
	'Load the level in memory
	Get #f, p, a
	a = Mid(a, Len(t) + 1)
	n = InStr(a, Chr(255))
	
	'Add a wait so splash message is not erased
	a = Left(a, n) + Chr(1) + MkShort(200)
	Close f
	
	Return a
End Function


Sub LoadOrSave(which As Byte)
	Dim f As Integer, sg(1 To 8) As SaveGame
	Dim As Short i, j, temp
	Dim corrupt As Byte, q As Byte
	Dim As String s, s2
	Dim As Short checksum, checksum2
	Dim t As Double, imtemp As UByte Ptr
	Dim jb As Integer, jx As Single, jy As Single, jlock As Byte
	
	f = FreeFile
	imtemp = ImageCreate(48, 48)
	
	'Load saved game descriptors
	For i = 1 To 8
		s = "syderal" + Chr(48 + i) + ".sav"
		If FileExists(s) Then
			Open s For Binary Access Read As f
			s2 = Space(3)
			Get #f, 1, s2
			If s2 <> "JSS" Then corrupt = -1
			Get #f, , sg(i).score
			Get #f, , sg(i).lives
			Get #f, , sg(i).level
			s2 = Space(Lof(f) - 3)
			Get #f, 4, s2
			checksum = CvShort(Mid(s2, 9, 2))
			Mid(s2, 9, 2) = MkShort(0)
			checksum2 = 0
			For j = 1 To Len(s2)
				checksum2 = (checksum2 + ASC(Mid(s2, j, 1))) Mod 21517
			Next j
			If checksum2 <> checksum Then corrupt = -1
			Close f
			
			checksum = 0 : checksum2 = 0
			If corrupt Then
				Kill s
				sg(i).score = 0
				sg(i).lives = 0
				sg(i).level = 0
			End If
		End If
	Next i
	
	PalGrade 0
	Put (0, 0), book(B_MLEFT).p, PSet
	Put (240, 0), book(SKY_IMAGE).p, PSet
	If English Then
		TextDisp 384, 54, "LEVEL  LIVES     SCORE", 1
	Else
		Put (384, 54), book(LEVEL_SCORE_LIVES).p, PSet
	End If
	PalGrade 1
	
	For i = 0 To 7
		t = Timer
		Do : Loop Until Timer >= t + .05
		
		If which = 0 Then	'Load
			If sg(i + 1).level Then
				Get book(DISKS).p, (0, 48)- Step (47, 47), imtemp
			Else
				Get book(DISKS).p, (0, 0)- Step (47, 47), imtemp
			End If
		Else	'Save
			If iscd Then
				Get book(DISKS).p, (0, 0)- Step (47, 47), imtemp
			Else
				Get book(DISKS).p, (0, 96)- Step (47, 47), imtemp
			End If
		End If
		Put (315, 120 + i * 54), imtemp, PSet
		
		DisplayFigure 474, 129 + i * 54, sg(i + 1).level
		DisplayFigure 627, 129 + i * 54, sg(i + 1).lives
		DisplayFigure 900, 129 + i * 54, sg(i + 1).score
	Next i
	
	Do : Loop Until Len(InKey) = 0
	
	Dim As Short current, ex
	Dim akey As String, disknumber As Byte
	
	current = 0 : ex = 1
	If which = 0 Then
		For i = 0 To 7
			If sg(i + 1).level Then current = i : Exit For
		NExt i
		If current = 1 Then ex = 0
	End If
	
	Do
		If current <> ex Then
			If which = 0 Then
				If sg(ex + 1).level Then disknumber = 1 Else disknumber = 0
			Else
				If iscd Then disknumber = 0 Else disknumber = 2
			End If
			Get book(DISKS).p, (0, 48 * disknumber)- Step (47, 47), imtemp
			Put (315, 120 + ex * 54), imtemp, PSet
			
			If which = 0 Then
				If sg(current + 1).level Then disknumber = 3 Else disknumber = 0
			Else
				disknumber = 4
			End If
			Get book(DISKS).p, (0, 48 * disknumber)- Step (47, 47), imtemp
			Put (315, 120 + current * 54), imtemp, PSet
			
			ex = current
		End If
		
		akey = InKey
		If joystick.use Then
			GetJoystick joystick.id, jb, jx, jy
			
			If jy < -.5 Then
				If jlock = 0 Then
					akey = Chr(255) + Chr(72)
					jlock = -1
				End If
			ElseIf jy > .5 Then
				If jlock = 0 Then
					akey = Chr(255) + Chr(80)
					jlock = -1
				End If
			ElseIf jy >= -.5 And jy <= .5 Then
				jlock = 0
			End If
			If jb Then
				If jb And 2 ^ joystick.action Then
					akey = Chr(13)
				ElseIf jb And 2 ^ joystick.back Then
					akey = Chr(27)
				End If
			End If
		End If
		
		Select Case	akey
			Case Chr(27) : current = -1 : Exit Do
			Case Chr(13)
				If iscd = 0 Or which = 0 Then Exit Do
			Case Chr(255) + Chr(72)
				If which Then
					current -= 1
					If current = -1 Then current = 7
				Else
					q = 0 : temp = current
					For j = current - 1 To 0 Step -1
						If sg(j + 1).level Then
							q = -1 : temp = j : Exit For
						End If
					Next j
					
					If Not q Then
						For j = 7 To current Step -1
							If sg(j + 1).level Then
								q = -1 : temp = j : Exit For
							End If
						Next j
					End If
					
					current = temp
				End If
			Case Chr(255) + Chr(80)
				If which Then
					current += 1
					If current = 8 Then current = 0
				Else
					q = 0 : temp = current
					For j = current + 1 To 7
						If sg(j + 1).level Then
							q = -1 : temp = j : Exit For
						End If
					Next j
					
					If Not q Then
						For j = 0 To current
							If sg(j + 1).level Then
								q = -1 : temp = j : Exit For
							End If
						Next j
					End If
					
					current = temp
				End If
			Case Chr(255) + "k"
				currentm = 255
				Exit Do
		End Select
	Loop
	
	If current <> -1 Then
		temp = current
		s = "syderal" + Chr(49 + temp) + ".sav"
		
		If which Then
			If Not iscd Then SaveTheGame s
		Else
			If sg(current + 1).level Then
				LoadTheGame s
				canscore = 0	'May not record a highscore till new game
			End If
		End IF
	End If
	
	PalGrade 2
	CLS
	PalGrade 3
	
	ImageDestroy imtemp
End Sub


Sub LoadTheGame(s As String)
	Dim f As Integer, i As Byte, num As String
	Dim levelon As Short
	
	f = FreeFile
	Open s For Binary Access Read As f
	Get #f, 14, levptr
	Get #f, , levmem
	Get #f, , player
	For i = 1 To 20
		Get #f, , enemy(i)
	Next i
	For i = 1 To 20
		Get #f, , beam(i)
	Next i
	For i = 1 To 20
		Get #f, , ani(i)
	Next i
	Close f
	
	playing = -1
	levelon = ((player.level - 1) Mod levels) + 1
	num = Trim(Str(levelon))
	num = String(3 - Len(num), "0") + num
	curlev = LoadLevel(num)
End Sub


Function Menu As Short
	Dim todemo As Double	'Time to go to demo
	Dim ex As Short, action As Byte = 0, akey As String
	Dim mimage As UByte Ptr, jlock As UByte = 0
	Dim jb As Integer, jx As Single, jy As Single
	
	'Empty keyboard buffer
	Do : Loop Until Len(InKey) = 0
	
	mimage = ImageCreate(483, 90)
	
	CLS
	PalGrade 0
	Put (0, 0), book(MENU_LEFT).p, PSet
	Put (240, 0), book(MENU_TOP).p, PSet
	Put (240, 309), book(MENU_BOTTOM).p, PSet
	PalGrade 1
	
	todemo = Timer + 10		'If 10 seconds idle, go to demo
	
	If currentm = 0 Then currentm = 1
	Do
		If ex <> currentm Then
			Put (240, 0), book(MENU_TOP).p, PSet
			Put (240, 309), book(MENU_BOTTOM).p, PSet
			Get book(MENU_OPTIONS).p, (0, 90 * (currentm - 1))- Step (482, 89), mimage
			Put (456, 66 + 87 * currentm), mimage, PSet
			If playing Then Put (650, 180), book(SHIP_ICON).p, PSet
			ex = currentm
		End If
		
		If action Then	'As long as not idle, stay in the menu
			action = 0
			todemo = Timer + 10
		End If
		
		akey = InKey
		If joystick.use Then
			GetJoystick joystick.id, jb, jx, jy
			
			If jy < -.5 Then
				If jlock = 0 Then
					akey = Chr(255) + Chr(72)
					jlock = -1
				End If
			ElseIf jy > .5 Then
				If jlock = 0 Then
					akey = Chr(255) + Chr(80)
					jlock = -1
				End If
			ElseIf jy >= -.5 And jy <= .5 Then
				jlock = 0
			End If
			If jb <> 0 Then akey = Chr(13)
		End If
		
		Select Case akey
			Case Chr(255) + Chr(72)
				If currentm > 1 Then currentm -= 1 : action = -1
			Case Chr(255) + Chr(80)
				If currentm < 4 Then currentm += 1 : action = -1
			Case Chr(13), " "
				Exit Do
			Case Chr(27)
				currentm = 0 : Exit Do
			Case Chr(255) + "k"		'Exit program, no matter what
				currentm = 4
				Exit Do
		End Select
		
		If Timer > todemo Then
			currentm = 100 + currentm
			Exit Do
		End If
	Loop
	
	ImageDestroy mimage
	
	PalGrade 2
	CLS
	PalGrade 3
	
	Return currentm
End Function


Sub MovePeople
	Dim As Byte i, j, k, q
	Dim As Short dif, xf, yf
	
	For i = 1 To 20
		If enemy(i).status <> UNUSED And enemy(i).status <> STANDBY Then
			Select Case enemy(i).behaviour
				Case 1	'P-Orb
					If Int(Rnd * 20) = 1 Then enemy(i).d = (enemy(i).d + 1) Mod 8
					Select Case enemy(i).d
						Case 0: enemy(i).x = enemy(i).x + 1
						Case 1: enemy(i).x = enemy(i).x + 1: enemy(i).y = enemy(i).y + 1
						Case 2: enemy(i).y = enemy(i).y + 1
						Case 3: enemy(i).y = enemy(i).y + 1: enemy(i).x = enemy(i).x - 1
						Case 4: enemy(i).x = enemy(i).x - 1
						Case 5: enemy(i).x = enemy(i).x - 1: enemy(i).y = enemy(i).y - 1
						Case 6: enemy(i).y = enemy(i).y - 1
						Case 7: enemy(i).y = enemy(i).y - 1: enemy(i).x = enemy(i).x + 1
					End Select
				Case 2	'H-Atom
					If Int(Rnd * 30) = 1 Then enemy(i).d = Int(Rnd * 10)
					Select Case enemy(i).d
						Case 0: enemy(i).x = enemy(i).x + 1
						Case 1: enemy(i).x = enemy(i).x + 1: enemy(i).y = enemy(i).y + 1
						Case 2: enemy(i).y = enemy(i).y + 1
						Case 3: enemy(i).y = enemy(i).y + 1: enemy(i).x = enemy(i).x - 1
						Case 4: enemy(i).x = enemy(i).x - 1
						Case 5: enemy(i).x = enemy(i).x - 1: enemy(i).y = enemy(i).y - 1
						Case 6: enemy(i).y = enemy(i).y - 1
						Case 7: enemy(i).y = enemy(i).y - 1: enemy(i).x = enemy(i).x + 1
					End Select
					If Int(Rnd * 400) = 1 Then enemy(i).status = ATTACK: enemy(i).position = 0
				Case 3	'MegaQuark
					Select Case enemy(i).d
						Case 0  'Right-Up
							If enemy(i).x = 300 Then
								enemy(i).d = 1
							ElseIf enemy(i).y = 0 Then
								enemy(i).d = 2
							Else
								enemy(i).x = enemy(i).x + 1
								enemy(i).y = enemy(i).y - 1
							End If
						Case 1  'Left-Up
							If enemy(i).x = 0 Then
								enemy(i).d = 0
							ElseIf enemy(i).y = 0 Then
								enemy(i).d = 3
							Else
								enemy(i).x = enemy(i).x - 1
								enemy(i).y = enemy(i).y - 1
							End If
						Case 2  'Right-Down
							If enemy(i).x = 300 Then
								enemy(i).d = 3
							ElseIf enemy(i).y = 159 Then
								enemy(i).d = 0
							Else
								enemy(i).x = enemy(i).x + 1
								enemy(i).y = enemy(i).y + 1
							End If
						Case 3  'Left-Down
							If enemy(i).x = 0 Then
								enemy(i).d = 2
							ElseIf enemy(i).y = 159 Then
								enemy(i).d = 1
							Else
								enemy(i).x = enemy(i).x - 1
								enemy(i).y = enemy(i).y + 1
							End If
					End Select
					If enemy(i).y > 140 And enemy(i).x < player.x + 16 And enemy(i).x + 19 > player.x + 4 Then player.nrg = 0
				Case 4	'Electron
					If enemy(i).d Then
						IF enemy(i).x > enemy(i).extra1 Then
							enemy(i).x = enemy(i).x - 1
						ElseIf enemy(i).x < enemy(i).extra1 Then
							enemy(i).x = enemy(i).x + 1
						ElseIf enemy(i).y > enemy(i).extra2 Then
							enemy(i).y = enemy(i).y - 1
						ElseIf enemy(i).y < enemy(i).extra2 Then
							enemy(i).y = enemy(i).y + 1
						Else
							enemy(i).d = 0
						End If
					ElseIf Int(Rnd * 30) = 1 Then
						enemy(i).d = 1
						enemy(i).extra1 = Int(Rnd * 300)
						enemy(i).extra2 = Int(Rnd * 140)
						enemy(i).status = ATTACK
						enemy(i).position = 0  'Always when in ATTACK mode
					End If
				Case 5	'Plutonium nucleus
					If Int(Rnd * 15) = 1 Then enemy(i).d = Int(Rnd * 10)
					Select Case enemy(i).d
						Case 1: enemy(i).y = enemy(i).y - 1
						Case 2: enemy(i).y = enemy(i).y + 1
					End Select
					
					q = 0
					For j = 1 To 20
						IF beam(j).status <> UNUSED And beam(j).d = UP Then
							dif = enemy(i).x - beam(j).x
							If beam(j).y > enemy(i).y And Abs(dif) < enemy(i).extra1 Then
								k = Sgn(dif)
								If k = 0 Then
									k = Int(Rnd * 2) + 1
									If k = 2 Then k = -1
								End If
								enemy(i).x = enemy(i).x + k
								q = -1
								Exit For
							End If
						End If
					Next j

					If Not q Then
						k = Sgn(player.x - enemy(i).x)
						enemy(i).x = enemy(i).x + k
					End If

					If Int(Rnd * 100) = 1 Then
						If enemy(i).x > player.x - 40 And enemy(i).x < player.x + 40 Then
							enemy(i).status = ATTACK
							enemy(i).position = 0
						End If
					End If
				Case 6
					If enemy(i).d Then
						xf = enemy(i).extra1
						yf = enemy(i).extra2
						If enemy(i).x = xf And enemy(i).y = yf Then
							enemy(i).d = 0
						Else
							For j = 1 To 2
								enemy(i).x = enemy(i).x + Sgn(xf - enemy(i).x)
								enemy(i).y = enemy(i).y + Sgn(yf - enemy(i).y)
							Next j
						End If
					End If

					If Int(Rnd * 75) = 7 Then
						enemy(i).extra1 = Int(Rnd * 300)
						enemy(i).extra2 = INT(Rnd * 140)
						enemy(i).d = 1
					End If

					If Int(Rnd * 100) = 1 Then
						If enemy(i).x > player.x - 40 And enemy(i).x < player.x + 40 Then
							enemy(i).status = ATTACK
							enemy(i).position = 0
						End If
					End If
		   
					If Int(Rnd * 3000) = 3 Then
						For j = 1 To 20
							If enemy(j).status = UNUSED Then enemy(j) = enemy(i) : Exit For
						NEXT j
					End If
			End Select
		   
			If enemy(i).behaviour <> 3 Then
				If enemy(i).x > 300 Then enemy(i).x = 300
				If enemy(i).x < 0 Then enemy(i).x = 0
				If enemy(i).y > 140 Then enemy(i).y = 140
				If enemy(i).y < 0 Then enemy(i).y = 0
			End If
		End If
	Next i
End Sub


Function NewMenu(curop As Byte) As Short
	Dim opimage As UByte Ptr, todemo As Double
	Dim t As Double, curoption As Byte
	Dim akey As String, i As Short
	Dim opindex(0 To 7) As Byte, firstloop As Byte = -1
	Dim jb As Integer, jx As Single, jy As Single, jlock As Byte
	
	curoption = curop
	
	opindex(0) = -1
	opindex(1) = 0
	opindex(4) = 3
	opindex(5) = 6
	opindex(6) = 7
	opindex(7) = 8
	If playing Then
		opindex(2) = 1
		opindex(3) = 2
	Else
		opindex(2) = 4
		opindex(3) = 5
	End If
	
	Do : Loop Until Len(InKey) = 0
	
	For i = 1 To 20
		star(i).x = Int(Rnd * 240) + 80
		star(i).y = Int(Rnd * 200)
		star(i).c = Int(Rnd * 14)
	Next i
	
	opimage = ImageCreate(480, 60)
	
	Put (0, 0), book(B_MLEFT).p, PSet
		
	t = Timer - .006
	todemo = Timer + 10
	Do
		If Timer > t + .006 Then
			ScreenLock
			Line (240, 0)-(959, 599), 17, BF
			For i = 1 To 20
				If star(i).y >= 199 Then
					star(i).y = 0
					star(i).x = Int(Rnd * 240) + 80
					star(i).c = Int(Rnd * 10) + 4
				Else
					star(i).y += 1
				End If
				Line (3 * star(i).x, 3 * star(i).y)- Step (2, 2), star(i).c, BF
			Next i
			For i = 1 To 7
				If English Then
					Get book(B_ENMENU).p, (0, 60 * opindex(i))- Step (479, 59), opimage
				Else
					Get book(B_SPMENU).p, (0, 60 * opindex(i))- Step (479, 59), opimage
				End If
				Put (390, 30 + 60 * i), opimage, Trans
			Next i
			Put (330, 36 + 60 * curoption), book(SHIP_ICON).p, PSet
			ScreenUnlock
			
			If firstloop Then
				PalGrade 1
				firstloop = 0
			End If
			
			SoundMaintain
			t = Timer
		End If
	
		akey = InKey
		If joystick.use Then
			GetJoystick joystick.id, jb, jx, jy
			
			If jy < -.5 Then
				If jlock = 0 Then
					akey = Chr(255) + Chr(72)
					jlock = -1
				End If
			ElseIf jy > .5 Then
				If jlock = 0 Then
					akey = Chr(255) + Chr(80)
					jlock = -1
				End If
			ElseIf jy >= -.5 And jy <= .5 Then
				jlock = 0
			End If
			If jb Then
				If (jb And 2 ^ joystick.action) Or (jb And 2 ^ joystick.start) Then
					akey = Chr(13)
				End If
			End If
		End If
		
		Select Case akey
			Case Chr(255) + Chr(72)
				curoption -= 1
				SoundEvent 8
				If curoption = 0 Then curoption = 7
			Case Chr(255) + Chr(80)
				curoption += 1
				SoundEvent 8
				If curoption = 8 Then curoption = 1
			Case Chr(255) + "k"
				curoption = 7
				Exit Do
			Case Chr(13)
				SoundEvent 4
				Exit Do
		End Select
		
		'See if we have to go the Demo
		If Len(akey) Then todemo = Timer + 10
		If Timer > todemo Then
			ImageDestroy opimage
			PalGrade 2 : CLS : PalGrade 3
			Return curoption + 100
		End If
	Loop
	
	ImageDestroy opimage
	PalGrade 2
	CLS
	PalGrade 3
	
	Return opindex(curoption)
End Function


Sub Options
	Dim akey As String, thisoption As Byte = 1, update As Byte
	Dim firsttime As Byte = -1, t As Double, i As Short
	Dim curcol As UByte = 57, curcoldir As Byte = 1
	Dim tsound As Double, butimage As UByte Ptr
	Dim As Single jx, jy : Dim jb As Integer, jlock As Byte
	Dim b(0 To 3) As Byte
	
	butimage = ImageCreate(45, 15)
		
	PalGrade 0
	Do : Loop Until Len(InKey) = 0
	
	update = -1 : t = Timer : tsound = Timer
	Do
		If Timer >= tsound + .005 Then
			SoundMaintain
			tsound = Timer
		End If
		
		If update Then
			ScreenLock
			Put (0, 0), book(B_MLEFT).p, PSet
			Put (240, 0), book(SKY_IMAGE).p, PSet
			If English Then
				Put (452, 24), book(B_ENOPTIONS_SIGN).p, Trans
			Else
				Put (426, 24), book(OPTIONS_SIGN).p, PSet
			End If
			Put (321, 240), book(VOLUME_SETUP).p, PSet
			If English Then Put (540, 243), book(B_ENVOLUME_TITLE).p, PSet
			Put (363 + 45 * volume, 270), book(LEVER_IMAGE).p, PSet
			Put (321, 330), book(BRIGHTNESS_SETUP).p, PSet
			If English Then Put (540, 333), book(B_ENBRIGHT_TITLE).p, PSet
			Put (363 + 45 * brightness, 360), book(LEVER_IMAGE).p, PSet
			Put (321, 420), book(JOYSTICK_BOX).p, PSet
			If English Then
				Put (627, 420), book(B_ENJBUTTONS).p, PSet
			Else
				Put (627, 420), book(B_JBUTTONBOX).p, PSet
			End If
			
			If joystick.use Then
				b(0) = joystick.action
				b(1) = joystick.start
				b(2) = joystick.back
				b(3) = joystick.deal
				
				For i = 0 To 3
					Get book(B_JBUTTONS).p, (0, 15 * b(i))- Step (44, 14), butimage
					Put (675, 456 + 21 * i), butimage, PSet
				Next i
			Else
				Put (429, 486), book(JOYSTICK_DELTA).p, PSet
			End If
			ScreenUnlock
			
			If firsttime Then PalGrade 1 : firsttime = 0
			update = 0
		End If
		
		If Timer >= t + .03 Then
			t = Timer
			ScreenLock
			Select Case thisoption
				Case 1
					For i = 1 To 3
						Line (321 - i, 240 - i)-(890 + i, 311 + i), curcol, B
					Next i
				Case 2
					For i = 1 To 3
						Line (321 - i, 330 - i)-(890 + i, 401 + i), curcol, B
					Next i
				Case 3
					For i = 1 To 3
						Line (321 - i, 420 - i)-(608 + i, 551 + i), curcol, B
					Next i
				Case 4
					For i = 1 To 3
						Line (627 - i, 420 - i)-(890 + i, 551 + i), curcol, B
					Next i
			End Select
			ScreenUnlock
			
			curcol += curcoldir
			If curcol = 56 Or curcol = 63 Then curcoldir = -curcoldir
		End If
		
		akey = InKey
		If joystick.use Then
			GetJoystick joystick.id, jb, jx, jy
			
			If jx >= -.5 And jx <= .5 And jy >= -.5 And jy <= .5 And jb = 0 Then jlock = 0
			
			If jlock = 0 Then
				If jx > .5 Then
					akey = Chr(255) + Chr(77) : jlock = -1
				ElseIf jx < -.5 Then
					akey = Chr(255) + Chr(75) : jlock = -1
				ElseIf jy > .5 Then
					akey = Chr(255) + Chr(80) : jlock = -1
				ElseIf jy < -.5 Then
					akey = Chr(255) + Chr(72) : jlock = -1
				ElseIf jb And 2 ^ joystick.action Then
					akey = Chr(13) : jlock = -1
				ElseIf jb And 2 ^ joystick.back Then
					akey = Chr(27) : jlock = -1
				End If
			End If
		End If
		
		Select Case akey
			Case Chr(27) : Exit Do
			Case Chr(13)
				Select Case thisoption
					Case 1
						volume = 5 : update = -1
						SoundEvent 8
					Case 2
						brightness = 5
						SetPalette 0, -1
						PalGrade 3
						SoundEvent 8
						update = -1
					Case 3
						If joystick.id <> -1 Then
							SoundEvent 8
							joystick.use = Not joystick.use
							update = -1
						Else
							SoundEvent 6
						End If
					Case 4
						If joystick.id <> -1 Then
							Put (627, 420), book(B_JBUTTONBOX).p, PSet
							For i = 0 To 3
								b(i) = GrabJButton
								If b(i) = 0 Then b(0) = 0 : Exit For
								Get book(B_JBUTTONS).p, (0, 15 * (b(i) - 1))- Step (44, 14), butimage
								Put (675, 456 + 21 * i), butimage, PSet
							Next i
							If b(0) Then
								joystick.action = b(0) - 1
								joystick.start = b(1) - 1
								joystick.back = b(2) - 1
								joystick.deal = b(3) - 1
							End If
						Else
							SoundEvent 6
						End If
				End Select
			Case Chr(255) + Chr(72)
				thisoption -= 1
				If thisoption = 0 Then thisoption = 4
				If thisoption = 3 Then thisoption = 2
				SoundEvent 8
				update = -1
			Case Chr(255) + Chr(80)
				thisoption += 1
				If thisoption = 5 Then thisoption = 1
				SoundEvent 8
				update = -1
			Case Chr(255) + Chr(77)
				If thisoption = 1 Then
					If volume < 10 Then
						volume += 1
						update = -1
						SoundEvent 8
					End If
				ElseIf thisoption = 2 Then
					If brightness < 10 Then
						brightness += 1
						update = -1
						SetPalette 0, -1
						PalGrade 3
						SoundEvent 8
					End If
				ElseIf thisoption = 3 Then
					thisoption = 4
					update = -1
					SoundEvent 8
				End If
			Case Chr(255) + Chr(75)
				If thisoption = 1 Then
					If volume > 0 Then
						volume -= 1
						update = -1
						SoundEvent 8
					End If
				ElseIf thisoption = 2 Then
					If brightness > 0 Then
						brightness -= 1
						update = -1
						SetPalette 0, -1
						PalGrade 3
						SoundEvent 8
					End If
				ElseIf thisoption = 4 Then
					thisoption = 3
					update = -1
					SoundEvent 8
				End If
			Case Chr(255) + "k"
				currentm = 255
				Exit Do
		End Select
	Loop
	
	ImageDestroy butimage
	SaveConfig
	
	PalGrade 2
	CLS
	PalGrade 3
End Sub


Sub PalGrade(how As Byte)
	Dim As Short i, j, k
	Dim As UByte r, g, b
	Dim As Double t
	
	Select Case how
		Case 0		'Blackout
			For i = 0 To 255
				Palette i, 0, 0, 0
			Next i
		Case 1		'Fade in
			For i = 0 To 255
				Palette i, 0, 0, 0
			Next i
			t = Timer
			Do : Loop Until Timer > t + .03
			For k = 5 To 1 Step -1
				j = 0
				For i = 0 To 255
					r = pal[j] ShR k
					g = pal[j + 1] ShR k
					b = pal[j + 2] ShR k
					Palette i, r, g, b
					j += 3
				Next i
				t = Timer
				Do : Loop Until Timer > t + .05
			Next k
			j = 0
			For i = 0 To 255
				r = pal[j]
				g = pal[j + 1]
				b = pal[j + 2]
				Palette i, r, g, b
				j += 3
			Next i
			t = Timer
			Do : Loop Until Timer > t + .07
			For k = 1 To 2
				j = 0
				For i = 0 To 255
					r = pal[j] ShL k
					g = pal[j + 1] ShL k
					b = pal[j + 2] ShL k
					Palette i, r, g, b
					j += 3
				Next i
				t = Timer
				Do : Loop Until Timer > t + .08
			Next k
		Case 2		'Fade out
			For k = 2 To 1 Step -1
				j = 0
				For i = 0 To 255
					r = pal[j] ShL k
					g = pal[j + 1] ShL k
					b = pal[j + 2] ShL k
					Palette i, r, g, b
					j += 3
				Next i
				t = Timer
				Do : Loop Until Timer > t + .08
			Next k
			j = 0
			For i = 0 To 255
				r = pal[j]
				g = pal[j + 1]
				b = pal[j + 2]
				Palette i, r, g, b
				j += 3
			Next i
			t = Timer
			Do : Loop Until Timer > t + .07
			For k = 1 To 5
				j = 0
				For i = 0 To 255
					r = pal[j] ShR k
					g = pal[j + 1] ShR k
					b = pal[j + 2] ShR k
					Palette i, r, g, b
					j += 3
				Next i
				t = Timer
				Do : Loop Until Timer > t + .05
			Next k
			For i = 0 To 255
				Palette i, 0, 0, 0
			Next i
		Case 3		'Normal palette
			j = 0
			For i = 0 To 255
				r = pal[j] ShL 2
				g = pal[j + 1] ShL 2
				b = pal[j + 2] ShL 2
				Palette i, r, g, b
				j += 3
			Next i
	End Select
End Sub


Sub Panel
	Put (0, 540), book(PANEL_IMAGE).p, PSet
	Put (690, 540), book(B_PATCH).p, PSet
End Sub


Sub Pause
	Dim subimage As UByte Ptr, frame As UByte Ptr, backframe As UByte Ptr
	Dim As Short current, pulse, d
	Dim t As Double, jb As Integer
	
	'Save background before placing the pause sign
	subimage = ImageCreate(543, 135)
	Get (210, 210)- Step (542, 134), subimage
	
	'Display pause sign
	#ifdef CLASSIC_STYLE
		Put (210, 210), book(PAUSE_BOX).p, PSet
	#else
		If English Then
			Put (297, 210), book(B_NEW_PAUSE).p, Trans
		Else
			Put (297, 210), book(B_NEW_PAUSA).p, Trans
		End If
	#endif
		
	'Initialise frame and direction of animation
	current = 0 : pulse = 0 : d = 0
	
	#ifdef CLASSIC_STYLE
		frame = ImageCreate(90, 90)
	#else
		frame = ImageCreate(360, 39)
		backframe = ImageCreate(360, 39)
		Get (297, 318)- Step (359, 38), backframe
	#endif

	If joystick.use Then
		Do
			GetJoystick joystick.id, jb
		Loop Until jb = 0
	End If

	Do
		If joystick.use Then
			GetJoystick joystick.id, jb
			If jb And 2 ^ joystick.start Then Exit Do
		End If
		
		'First check that "P" is released, then that it is pressed,
		'then that it is released again.
		Select Case pulse
			Case 0 : If MultiKey(25) = 0 Then pulse = 1
			Case 1 : If MultiKey(25) <> 0 Then pulse = 2
			Case 2 : If MultiKey(25) = 0 Then Exit Do
		End Select
		
		'Obtain animation frame
		#ifdef CLASSIC_STYLE
			Get book(PAUSE_ANI).p, (0, 90 * current)- Step (89, 89), frame
			Put (261, 234), frame, PSet
		#else
			Get book(B_NEW_PAUSE_ANI).p, (0, 39 * current)- Step (359, 38), frame
			Put (297, 318), backframe, PSet
			Put (297, 318), frame, Trans
		#endif
		
		t = Timer
		Do : Loop Until Timer >= t + .03
		
		#ifdef CLASSIC_STYLE
			If d Then
				current -= 1
				If current = -1 Then current = 0 : d = 0
			Else
				current += 1
				If current = 6 Then current = 5 : d = -1
			End If
		#else
			current = (current + 1) Mod 17
		#endif
	Loop
	
	If joystick.use Then
		Do
			GetJoystick joystick.id, jb
		Loop Until jb = 0
	End If
	
	Put (210, 210), subimage, PSet
	
	#ifndef CLASSIC_STYLE
		ImageDestroy backframe
	#endif
	ImageDestroy frame
	ImageDestroy subimage
End Sub


Function PlayGame(restart As Byte) As Byte
	Dim As Short exx, lastcycle
	Dim reason As Byte
	Dim lasttimer As Double, remember As Double
	Dim levelon As Short, lv As String
	Dim levelwait As Short, excuseme As Byte
	Dim As Byte pleft, pright, pshoot, ppause, pdeal, pup, pdown
	Dim jb As Integer, jx As Single, jy As Single
	Dim t As Double

	For i As Byte = 1 To 20
		star(i).x = Int(Rnd * 320)
		star(i).y = Int(Rnd * 160)
		star(i).c = Int(Rnd * 14)
	Next i

	If restart Then
		player.x = 150
		exx = 151 : player.d = 0
		player.score = 0
		player.head = 0
		player.nrg = 7
		player.shot = 70
		player.lives = 3
		player.record = 0
		player.level = 0
		player.cycle = 0
		
		curlev = ""		'Instruction buffer
		levptr = 1		'Pointer to current instruction
		levmem = 0		'Sleep time before executing next instruction
		
		For i As Byte = 1 To 20
			enemy(i).status = UNUSED
			ani(i).status = UNUSED
			beam(i).status = UNUSED
		Next i
	End If
	
	playing = -1
	lastcycle = player.cycle
	lasttimer = Timer
	
	Do
		CLS
		ShowScore -1
		Do
			t = Timer
			If InKey = Chr(255) + "k" Then currentm = 255 : Exit Do, Do
			
			'Every 500 points, player gets a life
			If player.head >= player.record + 500 And player.lives < 7 Then
				player.record = player.head
				player.lives += 1
				ShowScore -1
				SoundEvent 5
			End If
			
			player.cycle += 1
			If levmem > 0 Then levmem -= 1
			
			'player.shot is negative when charging and is used to count
			If player.shot <= 0 Then
				player.shot -= 1
				If player.shot = -1800 Then player.shot = 70
			End If
			
			'Have all data been executed for current level?
			If levptr > Len(curlev) Then
				player.level += 1	'Yes. Go to next level
				
				'Wrap around when all levels have been played
				'levelon is the actual level been loaded
				levelon = ((player.level - 1) Mod levels) + 1
				lv = Trim(Str(levelon))
				lv = String(3 - Len(lv), "0") + lv
				curlev = LoadLevel(lv)
				
				'Initialise new level
				player.shot = 70
				player.cycle = 0
				lastcycle = 0
				lasttimer = Timer
				levptr = 1
				'splash = -1
				
				If TestBeams Then
					levmem = 1500
					levelwait = -1
				Else
					levmem = 500
					splash = -1
					LevelSplash 1
				End If
			End If
			
			If levelwait <> 0 Then
				If TestBeams = 0 Then
					levmem = 500
					splash = -1
					'LevelSplash 1
					levelwait = 0
				End If
			End If
			
			'If levptr = 1 And levmem = 0 Then LevelSplash 0 : splash = 0
			If levptr = 1 And levmem = 0 Then splash = 0
			
			'Draw ship
			If player.x <> exx Then
				'Line (exx * 3, 480)-(exx * 3 + 59, 539), 0, BF
				'PutImage player.x, 160, d
				exx = player.x
				remember = Timer
			ElseIf player.d <> 0 And Timer - remember >= .05 Then
				'PutImage player.x, 160, 0
				player.d = 0
			End If
						
			'Initialise control detection
			pleft = 0 : pright = 0 : pshoot = 0 : ppause = 0 : pdeal = 0
			pup = 0 : pdown = 0
			
			'Check joystick
			If joystick.use Then
				GetJoystick joystick.id, jb, jx, jy
				
				If jx < -.5 Then
					pleft = -1
				ElseIf jx > .5 Then
					pright = -1
				End If
				
				If jy < 0 Then
					pup = -1
				ElseIf jy > 0 Then
					pdown = -1
				End If
				
				If jb And 2 ^ joystick.action Then pshoot = -1
				If jb And 2 ^ joystick.start Then ppause = -1
				If jb And 2 ^ joystick.deal Then
					If pup Then pdeal = 1
					If pdown Then pdeal = 2
				End If
				If jb And 2 ^ joystick.back Then reason = 2 : Exit Do, Do
			End If
			
			'Check keys
			If MultiKey(1) Then		'--- ESC
				If MultiKey(60) Then	'--- F2 + ESC (cheat)
					player.nrg = 1
				Else
					reason = 2
					Exit Do, Do
				End If
			End If
			If MultiKey(68) Then player.nrg = 0	'--- F10 (suicide)
			If MultiKey(25) Then ppause = -1	'--- P (Pause)
			If MultiKey(83) Then pdeal = 1		'--- DEL (give up a life)
			If MultiKey(80) Then pdeal = 2		'--- DOWN ARROW (buy NRG)
			IF MultiKey(77) Then pright = -1	'--- RIGHT ARROW
			If MultiKey(75) Then pleft = -1		'--- LEFT ARROW
			'--- SPACE BAR OR UP ARROW (shoot)
			If MultiKey(57) <> 0 Or MultiKey(72) <> 0 Then pshoot = -1

			
			'Actions
			If pright Then
				If player.x < 300 Then player.x += 1 : player.d = 1
			End If
			If pleft Then
				If player.x > 0 Then player.x -= 1 : player.d = 2
			End If
			If pshoot Then
				If excuseme <> 0 And player.shot > 0 And splash = 0 Then
					SoundEvent 2
					Shoot player.x, 140, UP, 1, EVERYBODY
					excuseme = 0
					player.shot -= 1
				End If
			End If
			If ppause Then
				Pause
				lastcycle = player.cycle
				lasttimer = Timer
			End If
			If pdeal Then		'Deal request made ("cheats")
				Select Case pdeal
					Case 1	'Give up a worn-up life
						If player.lives > 0 And (player.nrg < 7 Or player.shot < 70) Then
							player.lives -= 1
							player.nrg = 7
							player.shot = 70
							ShowScore -1
							SoundEvent 6
						End If
					Case 2	'Buy NRG by giving up a full ammo load
						#ifdef CLASSIC_STYLE
						If player.shot > 60 And player.nrg < 7 Then
						#else
						If player.shot > 40 And player.nrg < 7 Then
						#endif
							player.nrg += 1
							player.shot = 0
						End If
				End Select
			End If
						
			'Cannot just hold down the shooting key
			If pshoot = 0 Then excuseme = -1
		
			Cycle
			ShowScore 0
			
			If player.head < player.score Then player.head += .1
			
			'Sync to 200 cycles per second
			Do : Loop Until Timer >= t + .005
			
			If levmem = 0 And levptr <= Len(curlev) Then ExecI
		Loop Until player.nrg <= 0
		
		If player.nrg <= 0 Then
			StartAni player.x, 160, 30, 6, 12
			SoundEvent 1
			player.x = -50	'So shots aren't detected anymore
			
			Dim tp(768) As UByte, i As Short, which As Short, n As Short
			If bnw Then which = BNWPALETTE Else which = COLOURPALETTE
			
			For i = 0 To 767
				tp(i) = book(which).p[i]
			Next i
			
			For n = 1 To 600
				t = Timer
				Cycle
				
				If n > 400 Then
					For i = 0 To 767
						If tp(i) > n - 400 Then
							tp(i) -= (n - 400) \ 2
						Else
							tp(i) = 0
						End If
					Next i
					For i = 0 To 765 Step 3
						Palette i \ 3, tp(i) ShL 2, tp(i + 1) ShL 2, tp(i + 2) ShL 2
					Next i
				End If
				
				Do : Loop Until Timer >= t + .008
			Next n
			
			player.lives -= 1
			If player.lives = -1 Then		'Game Over!
				reason = 1: playing = 0
				If canscore Then
					LoadHighScore
					For i As Byte = 1 To 10
						If HighScore(i).score < player.score Then
							PushScore "", player.score, player.level
							Exit For
						End If
					Next i
				End If
				Do : Loop Until Len(InKey) = 0
				Exit Do
			End If
			player.x = 150 : player.d = 0
			exx = 151
			player.nrg = 7
			player.shot = 70
			lastcycle = 0
			lasttimer = Timer
			player.level -= 1			'These two lines force
			curlev = "" : levptr = 1	'level restart
			
			For i = 1 To 20
				enemy(i).status = UNUSED
				beam(i).status = UNUSED
			Next i
			CLS
			PalGrade 3
		End If
	Loop
	
	Return reason
End Function


Sub Present
	CLS		'Display "Dimioca Labs"
	PalGrade 0
	Put (120, 213), book(B_DIMILABS).p, PSet
	PalGrade 1
	Sleep 1500
	PalGrade 2
	
	CLS		'Display "SYDERAL"
	PalGrade 0
	SetPalette B_SPLASH_PALETTE, -1
	Put (90, 0), book(B_SPLASH).p, PSet
	PalGrade 1
	Sleep 1500
	PalGrade 2
	
	CLS		'Set empty screen with normal palette
	SetPalette 0, -1
	PalGrade 3
End Sub


Sub SaveConfig
	Dim As Integer f
	
	If iscd Then Exit Sub
	
	f = FreeFile
	Open "syderal.cfg" For Output As f
	If English Then
		Print #f, "brightness=";
	Else
		Print #f, "brillo=";
	End If
	Print #f, Trim(Str(brightness))
	If English Then
		Print #f, "b&w=";
		If bnw Then Print #f, "on" Else Print #f, "off"
	Else
		Print #f, "byn=";
		If bnw Then Print #f, "sí" Else Print #f, "no"
	End If
	Print #f, "video=";
	Select Case fullscreen
		Case -1 : If English Then Print #f, "fullscreen" Else Print #f, "completo"
		Case -2 : If English Then Print #f, "noborder" Else Print #f, "especial"
		Case Else : Print #f, "normal"
	End Select
	If English Then
		Print #f, "volume=";
	Else
		Print #f, "volumen=";
	End If
	Print #f, Trim(Str(volume))
	If joystick.use Then
		Print #f, "jaction=" + Trim(Str(joystick.action))
		Print #f, "jpause=" + Trim(Str(joystick.start))
		Print #f, "jdeal=" + Trim(Str(joystick.deal))
		Print #f, "jback=" + Trim(Str(joystick.back))
	Else
		Print #f, "joystick=ps"
	End If
	If English Then
		Print #f, "language=english"
	Else
		Print #f, "idioma=español"
	End If
	Close f
End Sub


Sub SaveHighScore
	Dim As Short i, j
	Dim f As Integer, s As String, b As UByte
	Dim w As Short, dw As Long
	
	If iscd Then Exit Sub
	
	f = FreeFile
	Open "syderal.hsc" For Output As f : Close f
	Open "syderal.hsc" For Binary As f
	s = ""
	For i = 1 To 256
		s &= Chr(Int(Rnd * 256))
	Next i
	Put #f, 1, s
	
	Seek #f, 9
	For i = 1 To 10
		s = Trim(HighScore(i).signature)
		b = Len(s) XOr 43
		Put #f, , b
		b = Int(Rnd * 256)
		For j = 1 To Len(s)
			Mid(s, j, 1) = Chr(ASC(Mid(s, j, 1)) XOr b XOr i)
		Next j
		Put #f, , s
		Put #f, , b
		dw = HighScore(i).score XOr &HFEDECACA
		Put #f, , dw
		w = HighScore(i).level XOr &HDED0 XOr (i ShL 1)
		Put #f, , w
		Seek #f, Seek(f) + 3
	Next i
	Close f
End Sub


Sub SaveTheGame(s As String)
	Dim f As Integer, s1 As String, i As Short
	Dim checksum As Short
	
	f = FreeFile
	Open s For Output As f : Close f
	Open s For Binary As f
	s1 = "JSS" + MkL(player.score) + MkShort(player.lives) + MkShort(player.level) + MkShort(0)
	Put #f, 1, s1
	Put #f, , levptr
	Put #f, , levmem
	Put #f, , player
	For i = 1 To 20
		Put #f, , enemy(i)
	Next i
	For i = 1 To 20
		Put #f, , beam(i)
	Next i
	For i = 1 To 20
		Put #f, , ani(i)
	Next i
	s1 = Space(LOF(f) - 3)
	Get #f, 4, s1
	checksum = 0
	For i = 1 To Len(s1)
		checksum = (checksum + ASC(Mid(s1, i, 1))) Mod 21517
	Next i
	Put #f, 12, checksum
	Close f
End Sub


Sub PushScore (sig As String, score As Long, level As Short)
	Dim As Byte i, j, csrblink, position, firstloop, accent, dier, tilde
	Dim t As Double, akey As String, cursig As String, s As String
	Dim jb As Integer, jx As Single, jy As Single, jtime As Double, jlock As Byte
	
	cursig = sig
	
	For i As Byte = 1 To 20
		star(i).x = Int(Rnd * 320)
		star(i).y = Int(Rnd * 200)
		star(i).c = Int(Rnd * 10) + 4
	Next i
	
	For i = 1 To 10
		If HighScore(i).score < score Then
			For j = 10 To i + 1 Step -1
				HighScore(j).signature = HighScore(j - 1).signature
				HighScore(j).score = HighScore(j - 1).score
				HighScore(j).level = HighScore(j - 1).level
			Next j
			position = i
			Exit For
		End If
	Next i
	
	firstloop = -1
	t = Timer - .006
	Do
		If Timer >= t + .006 Then
			ScreenLock
			Line (0, 0)-(959, 599), 0, BF
			For i = 1 To 20
				If star(i).y >= 199 Then
					star(i).y = 0
					star(i).x = Int(Rnd * 320)
					star(i).c = Int(Rnd * 10) + 4
				Else
					star(i).y += 1
				End If
				Line (3 * star(i).x, 3 * star(i).y)- Step (2, 2), star(i).c, BF
			Next i
			TextDisp 210, 210, "HAS ALCANZADO EL LUGAR " + Trim(Str(position))
			TextDisp 480 - 12 * Len(cursig), 288, cursig, 7
			csrblink = (csrblink + 1) Mod 100
			If csrblink > 50 Then Line (480 + 12 * Len(cursig), 288)- Step (11, 38), 15, BF
			ScreenUnlock
			
			If firstloop Then
				PalGrade 1
				firstloop = 0
			End If
			t = Timer
		End If
		
		akey = InKey
		If joystick.use Then
			GetJoystick joystick.id, jb, jx, jy
			
			If jx >= -.5 And jx <= .5 And jy >= -.5 And jy <= .5 Then
				If jb = 0 Then jlock = 0
			End If
			
			If jlock <> 0 And Timer >= jtime + .5 Then jlock = 0
			
			If jlock = 0 Then
				If jx > .5 Then
					akey = Chr(255) + Chr(77)
					jlock = -1 : jtime = Timer
				ElseIf jx < -.5 Then
					akey = Chr(255) + Chr(75)
					jlock = -1 : jtime = Timer
				ElseIf jy > .5 Then
					akey = Chr(255) + Chr(80)
					If Len(cursig) = 0 Then cursig = "A"
					jlock = -1 : jtime = Timer
				ElseIf jy < -.5 Then
					akey = Chr(255) + Chr(72)
					If Len(cursig) = 0 Then cursig = "A"
					jlock = -1 : jtime = Timer
				ElseIf jb And 2 ^ joystick.action Then
					akey = "A"
					jlock = -1 : jtime = Timer
				ElseIf jb And 2 ^ joystick.back Then
					akey = Chr(8)
					jlock = -1 : jtime = Timer
				ElseIf jb And 2 ^ joystick.start Then
					akey = Chr(13)
					jlock = -1 : jtime = Timer
				End If
			End If
		End If
				
		Select Case UCase(akey)
			Case "A" To "Z", "0" To "9", ".", " ", Chr(1) To Chr(7)
				If accent Then
					Select Case UCase(akey)
						Case "A" : akey = Chr(1)
						Case "E" : akey = Chr(2)
						Case "I" : akey = Chr(3)
						Case "O" : akey = Chr(4)
						Case "U" : akey = Chr(5)
						Case "N" : akey = Chr(7)
					End Select
				ElseIf dier Then
					If UCase(akey) = "U" Then akey = Chr(6)
				ElseIf tilde Then
					If UCase(akey) = "N" Then akey = Chr(7)
				End If
				If Len(cursig) < 12 Then cursig &= UCase(akey)
				accent = 0 : dier = 0 : tilde = 0
			Case "'" : accent = -1
			Case Chr(34) : dier = -1
			Case "~" : tilde = -1
			Case Chr(8)
				If Len(cursig) > 0 Then cursig = Left(cursig, Len(cursig) - 1)
			Case Chr(255) + Chr(72)
				If Len(cursig) Then
					s = Right(cursig, 1)
					If s = "Z" Then
						s = "."
					ElseIf s = "." Then
						s = " "
					ElseIf s = " " Then
						s = "0"
					ElseIf s = "9" Then
						s = "A"
					ElseIf s = Chr(7) Then
						s = "0"
					Else
						s = Chr(ASC(s) + 1)
					End If
					cursig = Left(cursig, Len(cursig) - 1) + s
				End If
			Case Chr(255) + Chr(80)
				If Len(cursig) Then
					s = Right(cursig, 1)
					If s = "." Then
						s = "Z"
					ElseIf s = "A" Then
						s = "9"
					ElseIf s = "0" Then
						s = " "
					ElseIf s = " " Then
						s = "."
					ElseIf s = Chr(1) Then
						s = "9"
					Else
						s = Chr(ASC(s) - 1)
					End If
					cursig = Left(cursig, Len(cursig) - 1) + s
				End If
			Case Chr(255) + Chr(77)
				If Len(cursig) Then
					s = Right(cursig, 1)
					Select Case s
						Case "A" : s = Chr(1)
						Case "E" : s = Chr(2)
						Case "I" : s = Chr(3)
						Case "O" : s = Chr(4)
						Case "U" : s = Chr(5)
						Case "N" : s = Chr(7)
						Case Chr(1) : s = "A"
						Case Chr(2) : s = "E"
						Case Chr(3) : s = "I"
						Case Chr(4) : s = "O"
						Case chr(5) : s = Chr(6)
						Case Chr(6) : s = "U"
						Case Chr(7) : s = "N"
					End Select
					cursig = Left(cursig, Len(cursig) - 1) + s
				End If
			Case Chr(255) + Chr(75)
				If Len(cursig) Then
					s = Right(cursig, 1)
					Select Case s
						Case "A" : s = Chr(1)
						Case "E" : s = Chr(2)
						Case "I" : s = Chr(3)
						Case "O" : s = Chr(4)
						Case "U" : s = Chr(6)
						Case "N" : s = Chr(7)
						Case Chr(1) : s = "A"
						Case Chr(2) : s = "E"
						Case Chr(3) : s = "I"
						Case Chr(4) : s = "O"
						Case chr(5) : s = "U"
						Case Chr(6) : s = Chr(5)
						Case Chr(7) : s = "N"
					End Select
					cursig = Left(cursig, Len(cursig) - 1) + s
				End If
			Case Chr(13), Chr(27) : Exit Do
		End Select
	Loop
	
	If Len(cursig) = 0 Then cursig = "ALGUIEN"
	HighScore(position).signature = UCase(cursig) + Space(12 - Len(cursig))
	HighScore(position).score = score
	HighScore(position).level = level
	SaveHighScore
	
	PalGrade 2
	CLS
	PalGrade 3
End Sub


Sub PutImage(x As Short, y As Short, n As Short)
	Get book(SPRITES).p, (0, 60 * n)- Step (59, 59), minibox
	Put (3 * x, 3 * y), minibox, Trans
End Sub


Sub SayGoodbye(n As Short)
	enemy(n).status = UNUSED
	enemy(n).id = 0
	StartAni enemy(n).x, enemy(n).y, 8, 7, 8
	SoundEvent 7
End Sub


Sub SayHello(x As Short, y As Short, behaviour As UByte, id As Short, inistat As Long)
	For i As Byte = 1 To 20
		If enemy(i).status = UNUSED Then
			StartAni x, y, 8, 7, 8
			enemy(i).position = 56 '8x7 cycles
			enemy(i).status = STANDBY
			enemy(i).x = x
			enemy(i).y = y
			enemy(i).behaviour = behaviour
			enemy(i).id = id

			Select Case behaviour
				Case 1  'P-Orb
					enemy(i).d = Int(Rnd * 8)
					enemy(i).nfirst = 3
					enemy(i).afirst = 3
					enemy(i).nframes = 5
					enemy(i).aframes = 5
					enemy(i).nspeed = 16
					enemy(i).aspeed = 16
					enemy(i).weapon = 0
					enemy(i).prize = 2
				Case 2  'H-Atom
					enemy(i).d = Int(Rnd * 10)
					enemy(i).nfirst = 15
					enemy(i).afirst = 15
					enemy(i).nframes = 10
					enemy(i).aframes = 10
					enemy(i).nspeed = 8
					enemy(i).aspeed = 8
					enemy(i).weapon = 2
					enemy(i).prize = 5
				Case 3  'MegaQuark
					If inistat >= 0 And inistat <= 3 Then
						enemy(i).d = inistat
					Else
						enemy(i).d = Int(Rnd * 4)
					End If
					enemy(i).nfirst = 43
					enemy(i).afirst = 43
					enemy(i).nframes = 6
					enemy(i).aframes = 6
					enemy(i).nspeed = 8
					enemy(i).aspeed = 8
					enemy(i).weapon = 0
					enemy(i).prize = 8
				Case 4  'Photon
					enemy(i).d = 0
					enemy(i).nfirst = 50
					enemy(i).afirst = 57
					enemy(i).nframes = 8
					enemy(i).aframes = 4
					enemy(i).nspeed = 8
					enemy(i).aspeed = 8
					enemy(i).weapon = 3
					enemy(i).prize = 12
				Case 5  'Plutonium nucleus
					enemy(i).d = Int(Rnd * 8)
					enemy(i).nfirst = 66
					enemy(i).afirst = 66
					enemy(i).nframes = 5
					enemy(i).aframes = 5
					enemy(i).nspeed = 16
					enemy(i).aspeed = 16
					enemy(i).weapon = 4
					enemy(i).prize = 20
					enemy(i).extra1 = Int(Rnd * 60) + 60
				Case 6  'Graviton
					enemy(i).d = 0  'Not moving
					enemy(i).nfirst = 78
					enemy(i).afirst = 96
					enemy(i).nframes = 18
					enemy(i).aframes = 3
					enemy(i).nspeed = 20
					enemy(i).aspeed = 12
					enemy(i).weapon = 5
					enemy(i).prize = 10
					enemy(i).extra3 = Int(Rnd * 18)   'Entry colour (?)
			End Select
			SoundEvent 4
			Exit For
		End If
	Next i
End Sub


Sub SetPalette(whichpal As Short, intensity As Short)
	Dim which As Short, i As Short, value As Long
	Dim ity As Short
	
	If whichpal Then
		which = whichpal
	ElseIf bnw Then
		which = BNWPALETTE
	Else
		which = COLOURPALETTE
	End If
	
	If intensity < 0 Then
		'Set to "brightness" setup option
		ity = 3 * (brightness ^ 2) + 25
	Else
		ity = intensity
	End If
	
	If ity = 100 Then
		'Set to untouched current palette
		For i = 0 To 767
			pal[i] = book(which).p[i]
		Next i
	ElseIf ity >= 0 And ity < 100 Then
		'Set to a darker shade
		For i = 0 To 767
			pal[i] = (book(which).p[i] * ity) \ 100
		Next i
	ElseIf ity > 100 Then
		'Set to a brighter shade and saturate
		For i = 0 To 767
			value = (book(which).p[i] * ity) \ 100
			If value > 63 Then value = 63
			pal[i] = value
		Next i
	End If
End Sub


Sub Shoot(x As Short, y As Short, d As Byte, kind As Short, victim As Short)
	For i As Byte = 1 To 20
		If beam(i).status = UNUSED Then
			beam(i).x = x
			beam(i).y = y
			beam(i).d = d
			beam(i).status = Not UNUSED
			Select Case kind
				Case 1
					beam(i).firstframe = 25
					beam(i).lastframe = 29
					beam(i).anispeed = 8
					beam(i).shotspeed = 2
					beam(i).damage = 1
				Case 2
					beam(i).firstframe = 36
					beam(i).lastframe = 42
					beam(i).anispeed = 10
					beam(i).shotspeed = 2
					beam(i).damage = 1
				Case 3
					beam(i).firstframe = 61
					beam(i).lastframe = 64
					beam(i).anispeed = 10
					#ifdef CLASSIC_STYLE
					beam(i).shotspeed = 1
					#else
					beam(i).shotspeed = 2
					#endif
					beam(i).damage = 2
				Case 4
					beam(i).firstframe = 72
					beam(i).lastframe = 76
					beam(i).anispeed = 10
					beam(i).shotspeed = 1
					beam(i).damage = 1
				Case 5
					beam(i).firstframe = 98
					beam(i).lastframe = 98
					beam(i).anispeed = 10
					beam(i).shotspeed = 2
					beam(i).damage = 3
			End Select
			beam(i).position = 0
			beam(i).victim = victim
			Exit For
		End If
	Next i
End Sub


Sub ShowScore (checklives As Byte)
	Dim As Short i, n, rshot, rnrg
	Dim s As String, imtemp As UByte Ptr
	
	imtemp = ImageCreate(30, 30)
	
	'Draw lives
	If checklives Then
		Panel
		For i = 1 To player.lives
			Put (i * 45 + 288, 555), book(SHIP_ICON).p, PSet
		Next i
	End If
	
	'Draw score
	s = Trim(Str(Int(player.head)))
	s = String(6 - Len(s), ":") + s
	For i = 1 To 6
		n = ASC(Mid(s, i, 1)) - 48
		Get book(SCORE_DIGITS).p, (0, 30 * n)- Step (29, 29), imtemp
		Put (i * 30 - 15, 549), imtemp, PSet
	Next i
	
	ImageDestroy imtemp
	imtemp = ImageCreate(33, 39)
	
	'Draw beam power bar
	If player.shot >= 0 Then
		rshot = (70 - player.shot) \ 10
		Get book(SHOT_ICONS).p, (0, 39 * rshot)- Step (32, 38), imtemp
	Else
		rshot = (2100 + player.shot) \ 300
		Get book(SHOT_LOADING).p, (0, 39 * rshot)- Step (32, 38), imtemp
	End If
	Put (267, 552), imtemp, PSet
	
	'Draw energy bar
	rnrg = 7 - player.nrg
	Get book(NRG_ICONS).p, (0, 39 * rnrg)- Step (32, 38), imtemp
	Put (213, 552), imtemp, PSet
	
	ImageDestroy imtemp
End Sub


Sub SoundEvent (n As Short)
	Dim i As Byte, found As Byte = 0
	Dim sound_page As Short, min_ As Long
	
	'No sound during demos or when the volume is zero
	If demomode <> 0 Or volume = 0 Then Exit Sub
	
	'Decide sound page for event number
	Select Case n
		Case 1 : sound_page = SND_BOOM
		Case 2 : sound_page = SND_SHOO
		Case 3 : sound_page = SND_CLANG
		Case 4 : sound_page = SND_BUZZ
		Case 5 : sound_page = SND_ONEUP
		Case 6 : sound_page = SND_UOHOH
		Case 7 : sound_page = SND_UZZ
		Case 8 : sound_page = B_SND_CLAC
	End Select
	
	'Find an available sound slot
	For i = 1 To 10
		If fx(i).status = 0 Then found = i : Exit For
	Next i
	
	'If all channels are busy, stop the one that will be available
	'the soonest...
	If found = 0 Then
		min_ = 1000000
		For i = 1 To 10
			If fx(i).status < min_ Then
				found = i
				min_ = fx(i).status
			End If
		Next i
		
		#ifdef __FB_LINUX__
			SoundStop fx(i).id
			'ChannelInit fx(i).id
		#elseif defined (__FB_WIN32__)
			SoundStop fx(i).id
		#endif
	End If
	
	#ifdef __FB_LINUX__
		'Sound is played at 22050Hz. Syderal runs at 200Hz
		'Therefore 110.25 samples are played per game tick
		fx(found).status = book(sound_page).length \ 110 + 1
		PlaySound fx(found).id, book(sound_page).p, 0 + book(sound_page).length
	#elseif defined (__FB_WIN32__)
		fx(found).status = book(sound_page).length \ 110 + 1
		PlaySound_ fx(found).id, book(sound_page).p, 0 + book(sound_page).length
	#endif
End Sub


Sub SoundMaintain
	For i As Byte = 1 To 10
		If fx(i).status > 0 Then fx(i).status -= 1
	Next i
End Sub


Sub StartAni (x As Short, y As Short, firstframe As Short, frames As Short, speed As Short)
	For i As Byte = 1 To 20
		If ani(i).status = UNUSED Then
			ani(i).x = x
			ani(i).y = y
			ani(i).status = Not UNUSED
			ani(i).firstframe = firstframe
			ani(i).lastframe = firstframe + frames - 1
			ani(i).speed = speed
			ani(i).position = 0
			Exit For
		End If
	Next i
End Sub


Function TestBeams As Byte
	Dim q As Byte
	
	For i As Byte = 1 To 20
		If beam(i).status <> UNUSED Then q = -1 : Exit For
	Next i
	
	Return q
End Function


Sub TextDisp (x As Short, y As Short, text As String, col As UByte = 0)
	Dim s As String, f As UByte Ptr
	Dim i As Short, c As UByte, j As Short
	Dim offs As Short
	
	If x = -1 Then x = 600 - 12 * Len(text)
	
	f = ImageCreate(24, 39)
	s = UCase(text)
	
	For i = 1 To Len(s)
		Select Case Mid(s, i, 1)
			Case Chr(1) To Chr(7)
				c = ASC(Mid(s, i, 1)) + 11
			Case ".", "0" To "9", "A" To "Z"
				c = ASC(Mid(s, i, 1)) - 46
			Case Chr(128) To Chr(255)
				Select Case Mid(s, i, 2)
					Case "á", "Á" : c = 12
					Case "é", "É" : c = 13
					Case "í", "Í" : c = 14
					Case "ó", "Ó" : c = 15
					Case "ú", "Ú" : c = 16
					Case "ü", "Ü" : c = 17
					Case "ñ", "Ñ" : c = 18
					Case Else : c = 200
				End Select
				i += 1
			Case Else
				c = 1
		End Select
		
		If c < 47 Then
			Get book(B_SCORE_FONT).p, (24 * c, 0)- Step (23, 38), f
			
			If col Then
				For offs = 32 To 1279
					If f[offs] Then f[offs] += (col ShL 4)
				Next offs
			End If
			
			Put (x + j * 24, y), f, Trans
			j += 1
		End If
	Next i
	
	ImageDestroy f
End Sub
