' This files is part of:
' Syderal (space shooter game)
' Copyright (C) 2000, 2016-2017  Lucas Pedrosa

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, version 3 of the License.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.
Dim Shared alsa As Byte = -1

Type snd_pcm_t As Any Ptr
Type snd_pcm_hw_params_t As Any Ptr

Declare Function snd_pcm_open Lib "asound" Alias "snd_pcm_open" ( _
ByVal pcm As snd_pcm_t Ptr, _
ByVal device As Zstring Ptr, _
ByVal direction As Integer, _
ByVal mode As Integer) As Integer

Declare Function snd_pcm_close Lib "asound" Alias "snd_pcm_close" ( _
ByVal pcm As snd_pcm_t) As Integer

Declare Function snd_pcm_hw_free Lib "asound" Alias "snd_pcm_hw_free" ( _
ByVal pcm As snd_pcm_t) As Integer

Declare Function snd_pcm_prepare Lib "asound" Alias "snd_pcm_prepare" ( _
ByVal pcm As snd_pcm_t) As Integer

Declare Function snd_pcm_writei Lib "asound" Alias "snd_pcm_writei" ( _
ByVal pcm As snd_pcm_t, _
ByVal buffer As Any Ptr, _
ByVal size As Integer) As Integer

Declare Function snd_pcm_recover Lib "asound" Alias "snd_pcm_recover" ( _
ByVal pcm As snd_pcm_t, _
ByVal err As Integer, _
ByVal silent As Integer) As Integer

Declare Function snd_pcm_hw_params_malloc Lib "asound" Alias "snd_pcm_hw_params_malloc" ( _
ByVal hw As snd_pcm_hw_params_t Ptr) As Integer

Declare Function snd_pcm_hw_params_any Lib "asound" Alias "snd_pcm_hw_params_any" ( _
ByVal pcm As snd_pcm_t, _
ByVal hw As snd_pcm_hw_params_t) As Integer

Declare Function snd_pcm_hw_params_set_access Lib "asound" Alias "snd_pcm_hw_params_set_access" ( _
Byval pcm As snd_pcm_t, _
Byval hw As snd_pcm_hw_params_t, _
Byval mode As Integer) As Integer

Declare Function snd_pcm_hw_params_set_format Lib "asound" Alias "snd_pcm_hw_params_set_format" ( _
ByVal pcm As snd_pcm_t, _
ByVal hw As snd_pcm_hw_params_t, _
ByVal fmt As Integer) As Integer

Declare Function snd_pcm_hw_params_set_channels Lib "asound" Alias "snd_pcm_hw_params_set_channels" ( _
ByVal pcm As snd_pcm_t, _
ByVal hw As snd_pcm_hw_params_t, _
ByVal channels As Integer) As Integer

Declare Function snd_pcm_hw_params_set_rate_near Lib "asound" Alias "snd_pcm_hw_params_set_rate_near" ( _
ByVal pcm As snd_pcm_t, _
ByVal hw As snd_pcm_hw_params_t, _
ByVal lpRate As Integer Ptr, _
ByVal lpDir As Integer Ptr) As Integer

Declare Function snd_pcm_hw_params Lib "asound" Alias "snd_pcm_hw_params" ( _
ByVal pcm As snd_pcm_t, _
ByVal hw As snd_pcm_hw_params_t) As Integer

Declare Sub snd_pcm_hw_params_free Lib "asound" Alias "snd_pcm_hw_params_free" ( _
ByVal hw As snd_pcm_hw_params_t)

Declare Function snd_pcm_drop Lib "asound" Alias "snd_pcm_drop" ( _
ByVal pcm As snd_pcm_t) As Integer


Dim Shared As snd_pcm_hw_params_t Ptr _snd_hw
Dim Shared As Integer _snd_ret, _snd_numframes, _snd_period_size

Type SoundChannel As snd_pcm_t Ptr


Sub ChannelInit(ByVal channel As SoundChannel Ptr)
   Dim value As Integer
   Dim direction As Integer = 0
   
      _snd_period_size = 1
      
      _snd_ret = snd_pcm_open(channel, "default", 0, 0)

      snd_pcm_hw_params_malloc(@_snd_hw)
      _snd_ret = snd_pcm_hw_params_any(*channel,_snd_hw)
      _snd_ret = snd_pcm_hw_params_set_access(*channel, _snd_hw, 3)

      _snd_ret = snd_pcm_hw_params_set_format(*channel, _snd_hw, 1)
      _snd_ret = snd_pcm_hw_params_set_channels(*channel, _snd_hw, 1)

      value = 22050
      _snd_ret = snd_pcm_hw_params_set_rate_near(*channel, _snd_hw, @value, @direction)

      _snd_ret = snd_pcm_hw_params(*channel,_snd_hw)
      snd_pcm_hw_params_free (_snd_hw)
      snd_pcm_prepare(*channel)
End Sub


Sub PlaySound (channel As SoundChannel, soundbuffer As Any Ptr, buffersize As Integer)
   Dim bs As Integer
   
      bs = buffersize / _snd_period_size
      _snd_ret = snd_pcm_writei(channel, soundbuffer, bs)
      If _snd_ret = -32 Then	'Check for EPIPE
         _snd_ret = snd_pcm_recover(channel, _snd_ret, 1)

         If _snd_ret = 0 Then _snd_ret = snd_pcm_writei(channel, soundbuffer, bs)
      End If
End Sub


Sub SoundStop(channel As SoundChannel)
	_snd_ret = snd_pcm_drop(channel)
End Sub


Sub SoundClose(channel As SoundChannel)
	SoundStop channel
	_snd_ret = snd_pcm_close(channel)
End Sub
