' This files is part of:
' Syderal (space shooter game)
' Copyright (C) 2000, 2016-2017  Lucas Pedrosa

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, version 3 of the License.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.
#include "windows.bi"
#Include Once "crt/string.bi"
#include "win/mmsystem.bi"
'#define WAVE_MAPPER -1


type QueueWAVEHDR
   _wavehdr as WAVEHDR
   _next as QueueWAVEHDR ptr
end type

Type InternalSoundChannel
	_hWaveOut As HWAVEOUT
	cs As CRITICAL_SECTION
	doneWaveHdrs As QueueWAVEHDR Ptr
	waitingbytes As Integer
End Type

Type SoundChannel As HWAVEOUT

Dim Shared _ss_ic(1 To 16) As InternalSoundChannel, _ss_ics As Byte = 0


Sub dsp_callback StdCall (_hWaveOut as HWAVEOUT, msg as UINT, instance as DWORD, p1 as DWORD, p2 as DWORD)
	If msg = WOM_DONE Then
		Dim p As QueueWAVEHDR Ptr, i As Byte
		For i = 1 To _ss_ics
			If _ss_ic(i)._hWaveOut = _hWaveOut Then Exit For
		Next i

		If i <= _ss_ics Then
			p = cast(QueueWAVEHDR ptr, p1)
			EnterCriticalSection(@_ss_ic(i).cs)
			p->_next = _ss_ic(i).doneWaveHdrs
			_ss_ic(i).doneWaveHdrs = p
			LeaveCriticalSection(@_ss_ic(i).cs)
		End If
	End If
End Sub

Sub clean_unprepares(s As InternalSoundChannel)
	EnterCriticalSection(@s.cs)
	Do While (s.doneWaveHdrs)
		Dim p As QueueWAVEHDR Ptr
		p = s.doneWaveHdrs
		s.doneWaveHdrs = p->_next
		s.waitingBytes -= p->_wavehdr.dwBufferLength
		waveOutUnprepareHeader(s._hWaveOut, @p->_wavehdr, sizeof(WAVEHDR))
		Deallocate(p->_wavehdr.lpData)
		Deallocate(p)
	Loop
	LeaveCriticalSection(@s.cs)
end sub

Sub dsp_finalize(s As InternalSoundChannel)
	If s._hWaveOut Then
		waveOutPause(s._hWaveOut)
		waveOutReset(s._hWaveOut)
		clean_unprepares s
		waveOutClose(s._hWaveOut)
		s._hWaveOut = NULL
		DeleteCriticalSection(@s.cs)
	End If
End Sub

Sub ChannelInit(c As SoundChannel Ptr)
	Dim As WAVEFORMATEX wfx

	If *c Then
		Dim i As Byte
		For i = 1 To _ss_ics
			If _ss_ic(i)._hWaveOut = *c Then Exit For
		Next i
		dsp_finalize _ss_ic(i)
	End If

	memset(@wfx, 0, SizeOf(wfx))
	wfx.wFormatTag = WAVE_FORMAT_PCM
	wfx.nChannels = 1	'(channels)
	wfx.nSamplesPerSec = 22050	'(frequency)
	wfx.wBitsPerSample = 8	'(bits)
	wfx.nBlockAlign = wfx.nChannels * wfx.wBitsPerSample / 8
	wfx.nAvgBytesPerSec =  wfx.nBlockAlign * wfx.nSamplesPerSec
	wfx.cbSize = 0
	If waveOutOpen(c, WAVE_MAPPER, @wfx, Cast(DWORD, @dsp_callback), 0, CALLBACK_FUNCTION) <> MMSYSERR_NOERROR Then
		*c = NULL
	Else
		_ss_ics += 1
		_ss_ic(_ss_ics)._hWaveOut = *c
		waveOutRestart(*c)
		InitializeCriticalSection(@_ss_ic(_ss_ics).cs)
	End If
End Sub


Sub PlaySound_ (s As SoundChannel, soundBuffer As Any Ptr, buffersize As Integer)
	Dim i As Byte
	For i = 1 To _ss_ics
		If _ss_ic(i)._hWaveOut = s Then Exit For
	Next i

	If i <= _ss_ics Then
		Dim p As WAVEHDR Ptr = Allocate(SizeOf(QueueWAVEHDR))
		memset(p, 0, SizeOf(WAVEHDR))
		p->lpData = Allocate(buffersize)

		If p->lpData = 0 Then
			Deallocate p
		Else
			memcpy(p->lpData, SoundBuffer, buffersize)
			p->dwBufferLength = buffersize: p->dwBytesRecorded = buffersize
			waveOutPrepareHeader(s, p, SizeOf(WAVEHDR))
			If waveOutWrite(s, p, SizeOf(WAVEHDR)) = MMSYSERR_NOERROR Then
				_ss_ic(i).waitingbytes += p->dwBufferLength
			Else
				waveOutUnprepareHeader(s, p, SizeOf(WAVEHDR))
				Deallocate p->lpData
				Deallocate p
			End If
		End If
		clean_unprepares _ss_ic(i)
	End If
End Sub


Sub SoundStop(s As SoundChannel)
	Dim i As Byte
	For i = 1 To _ss_ics
		If _ss_ic(i)._hWaveOut = s Then
			dsp_finalize _ss_ic(i)
			Exit For
		End If
	Next i
End Sub
